/*
    Copyright Analog Flavor 2013-2024

    All rights reserved.

    This software contains proprietary and confidential information which is the
    property of Analog Flavor and is subject to license terms.

    This file may be used in accordance with the license agreement 
    provided with the software.

    This copyright notice may not be removed or altered.
*/


#ifndef __af_return_codes__
#define __af_return_codes__

/*! \file af_return_codes.h
    \brief This file contains enum type definitions for the return codes used by the BeSpice interfaces.
*/

#ifndef AF_BSP_API
    #if defined _WIN32 || defined __CYGWIN__
        #ifdef AF_BSP_DLL_EXPORTS /* defined if we are building the DLL (instead of using it) */
            #define AF_BSP_API __declspec(dllexport)
        #else
            #define AF_BSP_API __declspec(dllimport)
        #endif
    #else
        #ifdef AF_BSP_DLL_EXPORTS /* defined if we are building the DLL (instead of using it) */
            #if __GNUC__ >= 4
                #define AF_BSP_API __attribute__ ((visibility("default")))
            #else
                #define AF_BSP_API
            #endif
        #else
            #define AF_BSP_API
        #endif
    #endif
#endif

/** This enum type defines the value returned for some of the functions defined
    in af_parser.h. */
typedef enum {
    af_code_ok = 0,                 /**< the function call succeeded */
    af_code_error = 1,              /**< the function call failed */
    af_code_file_not_found = 2,     /**< the file specified in the function call was not found */
    af_code_parse_failed = 3,       /**< the parsing of the file failed */
    af_code_licence_expired = 4,    /**< your licence expired */
    af_code_bad_parser_index = 5,   /**< the parser index is not valid */
    af_code_bad_name = 6,           /**< the name passed in the function call was not found */
    af_code_bad_selection = 7,      /**< the current selection is not appropriate for the function call */
    af_code_bad_index = 8,          /**< the index passed to the function is not valid */
    af_code_bad_function_arguments = 9,      /**< the function arguments are not valid */
    af_code_timeout = 10,           /**< a timeout occurred */
    af_code_queued = 11,            /**< the execution of the command was queued */
    af_code_bad_pointer = 12,       /**< an invalid pointer was passed to the function */
    af_code_thread_error = 13,      /**< the function was called from a different thread */
    af_code_interrupted = 14        /**< the action was interrupted by the user */
} af_return_code;

#endif

