/*
    Copyright Analog Flavor 2013-2024

    All rights reserved.

    This software contains proprietary and confidential information which is the
    property of Analog Flavor and is subject to license terms.

    This file may be used in accordance with the license agreement 
    provided with the software.

    This copyright notice may not be removed or altered.
*/


#ifndef __bspWaveWrapper__impl__
#define __bspWaveWrapper__impl__

#include "bspWaveWrapper.h"
#include <sstream>
#include <iomanip>
#include <stdlib.h>
#include <algorithm>

template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdGetStatus() {
    return cmdStd(std::string("get_status"));
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdShowControlPanel() {
    return cmdStd(std::string("show_control_panel 1"));
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdShowCommandLogger() {
    return cmdStd(std::string("show_command_logger 1"));
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdShowFileOpenDialog() {
    return cmdStd(std::string("show_file_open_dialog"));
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdWidgetParentsChanged() {
    return cmdStd(std::string("widget_parents_changed"));
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdHidePanels() {
    af_return_code rc_1, rc_2;
    rc_1 = cmdStd(std::string("show_control_panel 0"));
    rc_2 = cmdStd(std::string("show_command_logger 0"));
    if ( rc_2 == af_code_ok ) {
        return rc_1;
    }
    return rc_2;
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdCreatePlotIfEmpty() {
    int integer_result = 0;
    af_return_code rc = cmdStd("get_number_of_plots", integer_result);
    if ( rc == af_code_ok ) {
        if ( integer_result == 0 ) {
            return cmdStd("add_plot \"plot 1\" analog 0");
        }
    }
    return rc;
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdUnzoomPlot(const std::string &plot_name) {
    return cmdStd("zoom \"" + plot_name + "\" 0% 100% 0% 100% 0% 100%");
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdClearPlot(const std::string &plot_name) {
    return cmdStd("clear_plot \"" + plot_name + "\"");
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdCreateAnalogPlot(const std::string &plot_name, bool flag_new_tab, bool flag_switch_to_tab) {
    std::stringstream tmp;
    tmp << "add_plot \"" + plot_name + "\" analog";
    if ( flag_new_tab ) {
        tmp << " 1";
    }
    else {
        tmp << " 0";
    }
    if ( flag_switch_to_tab ) {
        tmp << " 1";
    }
    else {
        tmp << " 0";
    }
    return cmdStd(tmp.str());
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdCreateDigitalPlot(const std::string &plot_name, bool flag_switch_to_tab) {
    std::stringstream tmp;
    tmp << "add_plot \"" + plot_name + "\" digital";
    tmp << " 1";
    if ( flag_switch_to_tab ) {
        tmp << " 1";
    }
    else {
        tmp << " 0";
    }
    return cmdStd(tmp.str());
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdClosePlot(const std::string &plot_name) {
    return cmdStd("close_plot \"" + plot_name + "\"");
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdShowExportAsDataDialog(const std::string &plot_name) {
    return cmdStd("show_export_as_data_dialog \"" + plot_name + "\"");
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdShowExportAsImageDialog(const std::string &plot_name) {
    return cmdStd("show_export_as_image_dialog \"" + plot_name + "\"");
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdAddCurveToPlotByIdAndIndex(const std::string &plot_name, size_t section_id, size_t curve_index, bool flag_clear) {
    std::stringstream cmd;
    cmd << "add_curve_to_plot_by_id_and_index \""  << plot_name << "\" " << section_id << " " << curve_index << " ";
    if ( flag_clear ) {
        cmd << "1";
    }
    else {
        cmd << "0";
    }
    return cmdStd(cmd.str());
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdAddCurveToPlotByIndex(const std::string &plot_name, const std::string &section_name, size_t curve_index, bool flag_clear) {
    std::stringstream cmd;
    cmd << "add_curve_to_plot_by_index \""  << plot_name << "\" \"" << section_name << "\" " << curve_index << " ";
    if ( flag_clear ) {
        cmd << "1";
    }
    else {
        cmd << "0";
    }
    return cmdStd(cmd.str());
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdAddCurveToPlotByIndex(const std::string &plot_name, const std::string &section_name, size_t curve_index, const std::vector<size_t> &sweep_perm, const std::vector<size_t> &sweep_indices, bool flag_clear) {
    if ( sweep_perm.size() == 0 || sweep_indices.size() == 0 ) {
        return cmdAddCurveToPlotByIndex(plot_name, section_name, curve_index, flag_clear);
    }
    std::stringstream cmd;
    cmd << "add_n_dimensional_curve_to_plot_by_index \"" << plot_name << "\" \"" << section_name << "\" " << curve_index << " " << convertVectorToString(sweep_perm) << " " << convertVectorToString(sweep_indices) << " ";
    if ( flag_clear ) {
        cmd << "1";
    }
    else {
        cmd << "0";
    }
    return cmdStd(cmd.str());
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdRemoveCurveFromPlotByIndex(const std::string &plot_name, const std::string &section_name, size_t curve_index) {
    std::stringstream cmd;
    cmd << "remove_curve_from_plot_by_index \""  << plot_name << "\" \"" << section_name << "\" " << curve_index;
    return cmdStd(cmd.str());
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdRemoveCurveFromPlotByIndex(const std::string &plot_name, const std::string &section_name, size_t curve_index, const std::vector<size_t> &sweep_perm, const std::vector<size_t> &sweep_indices) {
    if ( sweep_perm.size() == 0 || sweep_indices.size() == 0 ) {
        return cmdRemoveCurveFromPlotByIndex(plot_name, section_name, curve_index);
    }
    std::stringstream cmd;
    cmd << "remove_n_dimensional_curve_from_plot_by_index \"" << plot_name << "\" \"" << section_name << "\" " << curve_index << " " << convertVectorToString(sweep_perm) << " " << convertVectorToString(sweep_indices);
    return cmdStd(cmd.str());
}

template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdHighlightCurveInPlotByIndex(const std::string &plot_name, const std::string &section_name, size_t curve_index, bool flag_clear) {
    std::stringstream cmd;
    cmd << "highlight_curve_in_plot_by_index \"" << plot_name << "\" \"" << section_name << "\" " << curve_index << " ";
    if ( flag_clear ) {
        cmd << "1";
    }
    else {
        cmd << "0";
    }
    return cmdStd(cmd.str());
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdHighlightCurveInPlotByIdAndIndex(const std::string &plot_name, size_t section_id, size_t curve_index, bool flag_clear) {
    std::stringstream cmd;
    cmd << "highlight_curve_in_plot_by_id_and_index \"" << plot_name << "\" " << section_id << " " << curve_index << " ";
    if ( flag_clear ) {
        cmd << "1";
    }
    else {
        cmd << "0";
    }
    return cmdStd(cmd.str());
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdHighlightCurveInPlotByIndex(const std::string &plot_name, const std::string &section_name, size_t curve_index, const std::vector<size_t> &sweep_perm, const std::vector<size_t> &sweep_indices, bool flag_clear) {
    if ( sweep_perm.size() == 0 || sweep_indices.size() == 0 ) {
        return cmdHighlightCurveInPlotByIndex(plot_name, section_name, curve_index, flag_clear);
    }
    std::stringstream cmd;
    cmd << "highlight_n_dimensional_curve_in_plot_by_index \"" << plot_name << "\" \"" << section_name << "\" " << curve_index << " " << convertVectorToString(sweep_perm) << " " << convertVectorToString(sweep_indices) << " ";
    if ( flag_clear ) {
        cmd << "1";
    }
    else {
        cmd << "0";
    }
    return cmdStd(cmd.str());
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdUnhighlightCurveInPlotByIndex(const std::string &plot_name, const std::string &section_name, size_t curve_index, bool flag_clear) {
    std::stringstream cmd;
    cmd << "unhighlight_curve_in_plot_by_index \"" << plot_name << "\" \"" << section_name << "\" " << curve_index << " ";
    if ( flag_clear ) {
        cmd << "1";
    }
    else {
        cmd << "0";
    }
    return cmdStd(cmd.str());
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdUnhighlightCurveInPlotByIndex(const std::string &plot_name, const std::string &section_name, size_t curve_index, const std::vector<size_t> &sweep_perm, const std::vector<size_t> &sweep_indices, bool flag_clear) {
    if ( sweep_perm.size() == 0 || sweep_indices.size() == 0 ) {
        return cmdUnhighlightCurveInPlotByIndex(plot_name, section_name, curve_index, flag_clear);
    }
    std::stringstream cmd;
    cmd << "unhighlight_n_dimensional_curve_in_plot_by_index \"" << plot_name << "\" \"" << section_name << "\" " << curve_index << " " << convertVectorToString(sweep_perm) << " " << convertVectorToString(sweep_indices) << " ";
    if ( flag_clear ) {
        cmd << "1";
    }
    else {
        cmd << "0";
    }
    return cmdStd(cmd.str());
}

template <typename IMPL_T>
std::string bspWaveWrapper<IMPL_T>::convertVectorToString(const std::vector<size_t> &vec) {
    std::stringstream cmd;
    cmd << "[";
    std::vector<size_t>::const_iterator it = vec.begin();
    if ( it != vec.end() ) {
        cmd << *it;
        it++;
        while ( it != vec.end() ) {
            cmd << "," << *it;
            it++;
        }
    }
    cmd << "]";
    return cmd.str();
}

template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdAddCurveToPlot(const std::string &plot_name, const std::string &section_name, const std::string &curve_name, bool flag_clear) {
    std::stringstream cmd;
    cmd << "add_curve_to_plot_by_name \"" << plot_name << "\" \"" << section_name << "\" \"" << curve_name << "\" ";
    if ( flag_clear ) {
        cmd << "1";
    }
    else {
        cmd << "0";
    }
    return cmdStd(cmd.str());
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdOpenFilesStd(const std::list<std::string> &files) {
    af_return_code rc = af_code_ok;
    af_return_code rc_single;
    for ( std::list<std::string>::const_iterator it = files.begin(); it != files.end(); it++ ) {
        rc_single = cmdOpenFileStd(*it);
        if ( rc_single != af_code_ok ) {
            rc = rc_single;
        }
    }
    return rc;
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdReloadFileStd(const std::string &file_name) {
    std::stringstream cmd;
    /// use quotes as the file name might contain whitespace characters
    cmd << "reload_file \"" << file_name << "\"";
    return cmdStd(cmd.str());
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdOpenFileStd(const std::string &file_name) {
    if (file_name.empty()) {
        return af_code_error;
    }
    std::stringstream cmd;
    /// use quotes as the file name might contain whitespace characters
    cmd << "open_file \"" << file_name << "\"";
    return cmdStd(cmd.str());
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdShowFileInTextEditor(const std::string &file) {
    std::stringstream cmd;
    /// use quotes as the file name might contain whitespace characters
    cmd << "show_file_in_text_editor \"" << file << "\"";
    return cmdStd(cmd.str());
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdCloseFileStd(const std::string &file_name) {
    std::stringstream cmd;
    /// use quotes as the file name might contain whitespace characters
    cmd << "close_file \"" << file_name << "\"";
    return cmdStd(cmd.str());
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdExecuteCommandFileStd(const std::string &file_name) {
    std::stringstream cmd;
    /// use quotes as the file name might contain whitespace characters
    cmd << "execute_command_file \"" << file_name << "\"";
    return cmdStd(cmd.str());
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::checkInstanceIdentifier(size_t id) {
    std::stringstream cmd;
    /// use quotes as the file name might contain whitespace characters
    cmd << "check_instance_identifier \"" << id << "\"";
    return cmdStd(cmd.str());
}
template <typename IMPL_T>
af_return_code  bspWaveWrapper<IMPL_T>::cmdStd(const std::string &cmd, std::vector<std::string> &string_results) {
    const char **bsp_result;
    const char *str;

    string_results.clear();
    af_return_code rc = cmdStd(cmd);

    if ( rc == af_code_ok ) {
        bsp_result = IMPL_T::getCommandResult();
        if ( bsp_result ) {
            while ( *bsp_result ) {
                str = *bsp_result;
                string_results.push_back(std::string(str));
                bsp_result++;
            }
        }
    }
    return rc;
}
template <typename IMPL_T>
af_return_code  bspWaveWrapper<IMPL_T>::cmdStd(const std::string &cmd, std::string &string_result) {
    const char **bsp_result;
    const char *str;

    string_result.clear();
    af_return_code rc = cmdStd(cmd);

    if ( rc == af_code_ok ) {
        bsp_result = IMPL_T::getCommandResult();
        if ( bsp_result ) {
            if ( *bsp_result ) {
                str = *bsp_result;
                string_result = std::string(str);
            }
        }
    }
    return rc;
}
template <typename IMPL_T>
af_return_code  bspWaveWrapper<IMPL_T>::cmdStd(const std::string &cmd, int &integer_result) {
    const char **bsp_result;
    const char *str;

    af_return_code rc = cmdStd(cmd);

    if ( rc == af_code_ok ) {
        rc = af_code_error;
        bsp_result = IMPL_T::getCommandResult();
        if ( bsp_result ) {
            if ( bsp_result[0] && !bsp_result[1] ) {
                str = *bsp_result;
                char *endptr;
                long int res = strtol(str, &endptr, 10);
                if ( endptr > str && *endptr == 0 ) {
                    integer_result = res;
                    rc = af_code_ok;
                }
            }
        }
    }
    return rc;
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdGetNbOfSections(int &nb_sections) {
    return cmdStd("get_number_of_sections", nb_sections);
}

template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdGetSectionName(int section_index, std::string &section_name) {
    std::stringstream cmd;
    cmd << "get_section_name " << section_index;
    return cmdStd(cmd.str(), section_name);
}

template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdGetWindowSizeHints(int &width, int &height) {
    if ( cmdStd("get_window_size_hint_width", width) == af_code_ok ) {
        return cmdStd("get_window_size_hint_height", height);
    }
    return af_code_error;
}


template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdGetSectionFileName(const std::string &section_name, std::string &file_name) {
    std::stringstream cmd;
    cmd << "get_section_file_name \"" << section_name << "\"";
    return cmdStd(cmd.str(), file_name);
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdAddCurveAnnotationToPlotByIdAndIndex(const std::string &plot_name, size_t section_id, size_t curve_index, const std::string &annotation_text, double x_coordinate) {
    std::stringstream cmd;
    cmd << std::setprecision(20);
    std::string txt = replaceDoubleQuotes(annotation_text);
    cmd << "add_curve_annotation_to_plot_by_id_and_index \"" << plot_name << "\" " << section_id << " " << curve_index << " point " << " \"" << txt << "\" " << x_coordinate;
    return cmdStd(cmd.str());
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdClearAnnotationsInPlot(const std::string &plot_name) {
    std::stringstream cmd;
    cmd << std::setprecision(20);
    cmd << "clear_annotations_in_plot \"" << plot_name << "\"";
    return cmdStd(cmd.str());
}
template <typename IMPL_T>
std::string bspWaveWrapper<IMPL_T>::replaceDoubleQuotes(const std::string &original_txt) const {
    size_t pos = original_txt.find('\"');
    if ( pos == std::string::npos ) {
        return original_txt;
    }
    std::string new_txt = original_txt;
    while ( pos < new_txt.size() ) {
        new_txt.replace(pos, 1, "\\\"");
        pos = pos + 2;
        pos = new_txt.find('\"', pos);
    }
    return new_txt;
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdStatusBarMessage(size_t severity, const std::string &msg) {
    std::stringstream cmd;
    std::string txt = replaceDoubleQuotes(msg);
    cmd << "show_status_bar_message " << severity << " \"" << txt << "\"";
    return cmdStd(cmd.str());
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdLoggerMessage(size_t severity, const std::string &msg) {
    std::stringstream cmd;
    std::string txt = replaceDoubleQuotes(msg);
    cmd << "show_logger_message " << severity << " \"" << txt << "\"";
    return cmdStd(cmd.str());
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdStatusBarProgress(size_t progress) {
    std::stringstream cmd;
    cmd << "show_status_bar_progress " << progress;
    return cmdStd(cmd.str());
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdDetachTab(const std::string &tab_name, bool flag_detach) {
    std::stringstream cmd;
    cmd << "detach_tab \"" << tab_name;
    if ( flag_detach ) {
        cmd << "\" 1";
    }
    else {
        cmd << "\" 0";
    }
    return cmdStd(cmd.str());
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdShowToolbarInTab(const std::string &tab_name, bool flag_show) {
    std::stringstream cmd;
    cmd << "show_toolbar_in_tab \"" << tab_name;
    if ( flag_show ) {
        cmd << "\" 1";
    }
    else {
        cmd << "\" 0";
    }
    return cmdStd(cmd.str());
}
template <typename IMPL_T>
void bspWaveWrapper<IMPL_T>::toLowerCase(std::string &s) {
    std::transform(s.begin(), s.end(), s.begin(), ::tolower);
}
template <typename IMPL_T>
void bspWaveWrapper<IMPL_T>::splitString(const std::string &txt, char separator, std::vector<std::string> &tokens) const {

    tokens.clear();

    size_t start_pos = 0;
    size_t end_pos = txt.find(separator, start_pos);
    std::string s;

    while ( end_pos != std::string::npos ) {
        s = txt.substr(start_pos, end_pos - start_pos);
        tokens.push_back(s);
        start_pos = end_pos;
        while ( start_pos < txt.size() && txt[start_pos] == separator ) {
            start_pos++;
        }
        end_pos = txt.find(separator, start_pos);
    }
    if ( start_pos < txt.size() ) {
        s = txt.substr(start_pos, txt.size());
        tokens.push_back(s);
    }
}
template <typename IMPL_T>
void bspWaveWrapper<IMPL_T>::replaceInString(std::string &s, const std::string &old_str, const std::string &new_str) const {
    size_t pos = s.find(old_str);
    if ( pos != std::string::npos ) {
        std::string new_s;
        while ( pos != std::string::npos ) {
            new_s = s.substr(0, pos);
            new_s = new_s + new_str;
            new_s = new_s + s.substr(pos + old_str.size());
            s = new_s;
            pos = s.find(old_str, pos + new_str.size());
        }
    }
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdSetOptionShowExitButton(bool flag_show) {
    if ( flag_show ) {
        return cmdStd(std::string("set_option show_exit_button 1"));
    }
    return cmdStd(std::string("set_option show_exit_button 0"));
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdSetOptionShowDigitalPlotButton(bool flag_show) {
    if ( flag_show ) {
        return cmdStd(std::string("set_option show_digital_plot_button 1"));
    }
    return cmdStd(std::string("set_option show_digital_plot_button 0"));
}

template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdSetOptionShowDetachButton(bool flag_show) {
    if ( flag_show ) {
        return cmdStd(std::string("set_option show_detach_button 1"));
    }
    return cmdStd(std::string("set_option show_detach_button 0"));
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdSetOptionShowAttachButton(bool flag_show) {
    if ( flag_show ) {
        return cmdStd(std::string("set_option show_attach_button 1"));
    }
    return cmdStd(std::string("set_option show_attach_button 0"));
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdSetOptionUseWhiteBackground(bool flag_white) {
    if ( flag_white ) {
        return cmdStd(std::string("set_option use_white_background 1"));
    }
    return cmdStd(std::string("set_option use_white_background 0"));
}
template <typename IMPL_T>
af_return_code bspWaveWrapper<IMPL_T>::cmdBringWindowToFront() {
    return cmdStd(std::string("bring_window_to_front 1"));
}
template <typename IMPL_T>
std::string bspWaveWrapper<IMPL_T>::convertReturnCodeToString(af_return_code rc) {
    switch (rc) {
    case af_code_ok: return "ok";
    case af_code_error: return "error";
    case af_code_file_not_found: return "file_not_found";
    case af_code_parse_failed: return "parse_failed";
    case af_code_licence_expired: return "license_expired";
    case af_code_bad_parser_index: return "bad_parser_index";
    case af_code_bad_name: return "bad_name";
    case af_code_bad_selection: return "bad_selection";
    case af_code_bad_index: return "bad_index";
    case af_code_bad_function_arguments: return "bad_function_arguments";
    case af_code_timeout: return "timeout";
    case af_code_queued: return "queued";
    case af_code_bad_pointer: return "bad_pointer";
    case af_code_thread_error: return "thread_error";
    case af_code_interrupted: return "interrupted";
    }
    return "";
}


#endif // __bspWaveWrapper__impl__
