/*
    Copyright Analog Flavor 2013-2024

    All rights reserved.

    This software contains proprietary and confidential information which is the
    property of Analog Flavor and is subject to license terms.

    This file may be used in accordance with the license agreement 
    provided with the software.

    This copyright notice may not be removed or altered.
*/

#ifndef __bspWaveWrapperCallbacks__h__
#define __bspWaveWrapperCallbacks__h__

#include "bspwave_widget.h"

#include <string>


/// bspWaveWrapperCallbacks simplifies the usage of callback functions.
///
/// These callback functions are called when the user oerforms a number of operations in BeSpice Wave.
/// They allow the integrating application to take appropriate actions.
///
/// To receive callbacks simply re-implement virtual functions such as
///     bspWaveWrapperCallbacks::onCurveHighlighted().
/// Callbacks are only active if callback functions have been registered.
/// For the Qt interface this means that qtBspWaveWrapper::registerCallbackFunctions() has been called.
class bspWaveWrapperCallbacks {


public:

    bspWaveWrapperCallbacks() {}

    /// if this function returns 1, the event is not processed by bespice wave
    virtual int onOpenFileButtonActivated() { return 0; }
    virtual void onCloseButtonActivated() {}
    virtual void onAttachButtonActivated() {}
    virtual void onDetachButtonActivated() {}

    virtual void onCurveHighlighted(const std::string &section_name, const std::string &curve_name, int section_id, int curve_index, int flag_clear, int from_browser);
    virtual void onCurveUnHighlighted(const std::string &section_name, const std::string &curve_name, int section_id, int curve_index, int flag_clear, int from_browser);
    virtual void onCurveActivated(const std::string &section_name, const std::string &curve_name, int section_id, int curve_index, int flag_clear, int from_browser);
    virtual void onCheckIntegratorLicence(void *data);
    // (remark) The string "received_text" might be modified.
    //      Change received_text to an empty string to prevent further actions.
    virtual bool onDropReceived(const std::string &plot_name, std::string &received_text);
    virtual bool onGenerateDragString(const std::string &xml_string, std::string &drop_string);
    virtual void onCursorValuesChanged(const std::string &plot_name, const char* const* cursor_values);

    virtual void onLoggerMessage(int severity, const std::string &message);
    virtual void onStatusBarMessage(int severity, const std::string &message);
    virtual void onProgressMessage(const std::string &message, int progress);
    virtual void onFileWasOpened(const std::string &file_name);
    virtual void onFileWasClosed(const std::string &file_name);
    virtual void onAllFilesWereClosed() {}
    virtual void onSectionUsedForLinkChanged() {}
    virtual int onIntegrationSpecificNotification1(const std::string &str, int i);
    virtual void onCurveToSchematicMappingDefined(const std::string &spice_instance, const std::string &curve_instance, int flag_simplify);
    virtual void onFocusIn() {}
    virtual void onFocusOut() {}
    virtual void onConfigurationChanged() {}
    virtual void onCopyHighlighting() {}

    virtual int onNotify(const struct _af_bspwave_callback_notification_type *description) {
        return handleNotification(description);
    }

protected:

    int handleNotification(const struct _af_bspwave_callback_notification_type *description);

};



#endif

