/*
    Copyright Analog Flavor 2013-2024

    All rights reserved.

    This software contains proprietary and confidential information which is the
    property of Analog Flavor and is subject to license terms.

    This file may be used in accordance with the license agreement 
    provided with the software.

    This copyright notice may not be removed or altered.
*/

#ifndef __bspwave_win_plugin__h__
#define __bspwave_win_plugin__h__

#include "bspwave_widget.h"
#include "af_wp_access_functions.h"

#ifdef __cplusplus
extern "C" {
#endif

    struct af_bspwave_win_plugin_type {

        void *plugin_object;

        /// The 1st argument (plugin_object) is the above callback_object.
        af_return_code (*get_plugin_property)(void *obj, int property_index, af_bspwave_plugin_property_type *property);
        af_return_code (*release_plugin)(void *plugin_object);
        HWND (*create_widget)(void *plugin_object, void *af_reserved, HWND parent, int page_index);

        /// The 2nd argument "created_widget" is the widget returned by the function "create_widget".
        af_return_code (*get_widget_property)(void *plugin_object, HWND created_widget, int property_index, af_bspwave_plugin_property_type *property);
        af_return_code (*execute_widget_action)(void *plugin_object, HWND created_widget, const af_bspwave_plugin_property_type *action);
        void (*register_widget_callback)(void *plugin_object,  HWND created_widget, void *obj, af_return_code(*execute_command)(void *obj, const char *command), const char **(*get_command_result)(void *obj));

        /// This function must be implemented if the plugin needs to access waveforms loaded in BeSpice Wave.
        ///     The access functions defined in parser_functions are passed to the plugin.
        void (*register_waveform_parser_functions)( void *plugin_object, HWND created_widget, af_wp_access_functions_type *parser_functions );

        /// This function is must be implemented if the plugin also implements a waveform parser.
        ///     This allows to visualize curves held in memory for example.
        ///     The function must fill the parser_functions functions.
        void(*register_waveform_parser)( void *plugin_object, HWND created_widget, struct _af_wp_plugin_functions_type *parser_functions );

    };


    AF_BSP_API af_return_code win_bspwave_register_plugin(HWND bspwave_widget_ptr, const af_bspwave_win_plugin_type *plugin);

#ifdef __cplusplus
}
#endif

#endif
