/*
    Copyright Analog Flavor 2013-2024

    All rights reserved.

    This software contains proprietary and confidential information which is the
    property of Analog Flavor and is subject to license terms.

    This file may be used in accordance with the license agreement 
    provided with the software.

    This copyright notice may not be removed or altered.
*/


#ifndef __bspwave_win_widget__h__
#define __bspwave_win_widget__h__

#include "bspwave_widget.h"

#ifdef __cplusplus
extern "C" {
#endif

    AF_BSP_API HWND win_bspwave_create_widget(HWND parent, int x, int y, int width, int height, int style_flags);

    AF_BSP_API af_return_code win_bspwave_process_command_line(HWND bspwave_hwnd, LPSTR lp_cmd_line);

    AF_BSP_API af_return_code win_bspwave_execute_command(HWND bspwave_hwnd, const char *command_name);
    AF_BSP_API const char **win_bspwave_get_command_result(HWND bspwave_hwnd);
    AF_BSP_API const char *win_bspwave_get_command_result_line(HWND bspwave_hwnd, int line_index);

    /// these commands must only be used in the thread that called "win_bspwave_create_widget" 
    AF_BSP_API af_return_code win_bspwave_show_widget(HWND bspwave_hwnd, int flag_show);
    AF_BSP_API af_return_code win_bspwave_reparent_widget(HWND bspwave_hwnd, HWND new_parent_hwnd);
    AF_BSP_API af_return_code win_bspwave_move_widget(HWND bspwave_hwnd, int x, int y, int width, int height);

    AF_BSP_API af_return_code win_bspwave_register_callback(HWND bspwave_hwnd, const af_bspwave_callback_type *callbacks);

    /// for accessing the waveform parser
    AF_BSP_API af_return_code win_bspwave_get_waveform_parser(HWND bspwave_hwnd, struct _af_wp_access_functions_type *functions);

    AF_BSP_API HWND win_bspwave_create_and_run_main(int width, int height, int style_flags, int flag_run_in_thread, const char *startup_commands);

    /// for accessing the child windows
    struct af_bspwave_win_widget_info_type {

        int m_info_type;
        int m_id;

        HWND m_widget_handle;
        int m_width;
        int m_height;
        HMENU m_menu_handle;
        int m_flag;

    };

    AF_BSP_API void win_bspwave_get_widget_info(HWND bspwave_hwnd, struct af_bspwave_win_widget_info_type *widget_info);

    struct af_bspwave_win_callback_type {

        void *callback_object;

        void(*tab_ctrl_page_added)(void*, const char *page_name, HWND page_handle);
        void(*tab_ctrl_page_removed)(void*, const char *page_name, HWND page_handle);
        void(*tab_ctrl_page_changed)(void*, const char *page_name, HWND page_handle);

        void(*ensure_page_visible)( void*, const char *page_name, HWND page_handle );
    };

    AF_BSP_API af_return_code win_bspwave_register_win_callback(HWND bspwave_hwnd, const struct af_bspwave_win_callback_type *callbacks);

    AF_BSP_API DWORD win_bspwave_handle_drag_enter(HWND bspwave_hwnd, HWND page_handle, LPDATAOBJECT pDataObject, DWORD dwKeyState);
    AF_BSP_API BOOL win_bspwave_handle_drop(HWND bspwave_hwnd, HWND page_handle, LPDATAOBJECT pDataObject, DWORD dropEffect);
    AF_BSP_API void win_bspwave_handle_drag_leave(HWND bspwave_hwnd, HWND page_handle);

    AF_BSP_API af_return_code win_bspwave_register_style_callback(HWND bspwave_hwnd, const struct af_win_widget_style_callback_type *callbacks);


    /* use these type definitions for ::GetProcAddress if the dll is loaded on runtime by ::LoadLibrary */
    typedef HWND win_bspwave_create_widget_type(HWND parent, int x, int y, int width, int height, int style_flags);
    typedef af_return_code win_bspwave_process_command_line_type(HWND bspwave_hwnd, LPSTR lp_cmd_line);
    typedef af_return_code win_bspwave_execute_command_type(HWND bspwave_hwnd, const char *command_name);
    typedef const char **win_bspwave_get_command_result_type(HWND bspwave_hwnd);
    typedef af_return_code win_bspwave_register_callback_type(HWND bspwave_hwnd, const af_bspwave_callback_type *callbacks);
    typedef HWND win_bspwave_create_and_run_main_type(int width, int height, int style_flags, int flag_run_in_thread, const char *startup_commands);
    typedef af_return_code win_bspwave_get_waveform_parser_type(HWND bspwave_hwnd, struct _af_wp_access_functions_type *functions);
    typedef void win_bspwave_get_widget_info_type(HWND bspwave_hwnd, struct af_bspwave_win_widget_info_type *widget_info);
    typedef af_return_code win_bspwave_register_win_callback_type(HWND bspwave_hwnd, const struct af_bspwave_win_callback_type *callbacks);
    typedef af_return_code win_bspwave_register_style_callback_type(HWND bspwave_hwnd, const struct af_win_widget_style_callback_type *callbacks);


#ifdef __cplusplus
}
#endif

#endif //__wxBspWaveWidget__

