/*
    Copyright Analog Flavor 2013-2024

    All rights reserved.

    This software contains proprietary and confidential information which is the
    property of Analog Flavor and is subject to license terms.

    This file may be used in accordance with the license agreement 
    provided with the software.

    This copyright notice may not be removed or altered.
*/


#ifndef __cliWinBspWaveWrapper__
#define __cliWinBspWaveWrapper__

//#include <windows.h>
//#include <vcclr.h>


using namespace System;
using namespace System::Windows;
using namespace System::Windows::Interop;
using namespace System::Runtime::InteropServices;

// forward declaration of unmanaged BeSpice Wave wrapper
class winBspWaveWrapperRuntimeWithCallback;

// interface used by callbacks from BeSpice Wave
public interface class cliBspWaveWrapperInterface {
    void onCloseButtonActivated();
    void onAttachButtonActivated();
    void onDetachButtonActivated();
    int onNotify(const struct _af_bspwave_callback_notification_type *description);
    void onCurveHighlighted(const String ^section_name, const String ^curve_name, int section_id, int curve_index, int flag_clear, int from_browser);
    void onCurveUnHighlighted(const String ^section_name, const String ^curve_name, int section_id, int curve_index, int flag_clear, int from_browser);
    void onCurveActivated(const String ^section_name, const String ^curve_name, int section_id, int curve_index, int flag_clear, int from_browser);
    void onCheckIntegratorLicence(void *data);
    bool onDropReceived(const String ^plot_name, String ^received_text);
    bool onGenerateDragString(const String ^xml_string, String ^drop_string);
    void onLoggerMessage(int severity, const String ^message);
    void onStatusBarMessage(int severity, const String ^message);
    void onProgressMessage(const String ^message, int progress);
    void onFileWasOpened(const String ^message);
    void onFileWasClosed(const String ^message);
    void onAllFilesWereClosed();
    void onSectionUsedForLinkChanged();
};

// managed BeSpice Wave wrapper
public ref class cliWinBspWaveWrapper : public HwndHost, public cliBspWaveWrapperInterface {

    winBspWaveWrapperRuntimeWithCallback *m_impl;
    int m_styleFlags;

public:

    cliWinBspWaveWrapper();
    cliWinBspWaveWrapper(const String ^library_path, int style_flags);
    ~cliWinBspWaveWrapper();

    winBspWaveWrapperRuntimeWithCallback *getImpl() { return m_impl; }

    // loads the dll at runtime and define the access functions for managing the BeSpice Wave Widget
    void loadBspWaveDll(const String ^library_path);
    void setStyleFlags(int style_flags) {
        m_styleFlags = style_flags;
    }
    HWND getBspwaveWidgetPtr();

    bool checkRequires64bLibrary() {
        int s = sizeof( void* );
        return ( s == 8 );
    }

    int cmdStd(String ^cmd);
    int cmdClose(bool flag_force);

    void registerCallbackFunctions(bool flag_drag_string_gene, bool flag_cursor_values);
    void unregisterCallbackFunctions();

    virtual void onCreated() {}

    // default interface implementation for BeSpice Wave callback Functions
    virtual void onCloseButtonActivated() {}
    virtual void onAttachButtonActivated() {}
    virtual void onDetachButtonActivated() {}
    virtual int onNotify(const struct _af_bspwave_callback_notification_type *description);
    virtual void onCurveHighlighted(const String ^section_name, const String ^curve_name, int section_id, int curve_index, int flag_clear, int from_browser) {}
    virtual void onCurveUnHighlighted(const String ^section_name, const String ^curve_name, int section_id, int curve_index, int flag_clear, int from_browser) {}
    virtual void onCurveActivated(const String ^section_name, const String ^curve_name, int section_id, int curve_index, int flag_clear, int from_browser) {}
    virtual void onCheckIntegratorLicence(void *data) {}
    virtual bool onDropReceived(const String ^plot_name, String ^received_text) {
        return false;
    }
    virtual bool onGenerateDragString(const String ^xml_string, String ^drop_string) {
        return false;
    }
    virtual void onLoggerMessage(int severity, const String ^message) {}
    virtual void onStatusBarMessage(int severity, const String ^message) {}
    virtual void onProgressMessage(const String ^message, int progress) {}
    virtual void onFileWasOpened(const String ^message) {}
    virtual void onFileWasClosed(const String ^message) {}
    virtual void onAllFilesWereClosed() {}
    virtual void onSectionUsedForLinkChanged() {}

protected:

    // Deallocate the native object on the finalizer just in case no destructor is called
    !cliWinBspWaveWrapper();

    // handles the creation of a new BeSpice Wave Widget
    virtual HandleRef BuildWindowCore(HandleRef hwndParent) override;
    virtual void DestroyWindowCore(HandleRef hwnd) override;


private:

};


#endif //__cliWinBspWaveWrapper__
