/*
    Copyright Analog Flavor 2013-2024

    All rights reserved.

    This software contains proprietary and confidential information which is the
    property of Analog Flavor and is subject to license terms.

    This file may be used in accordance with the license agreement 
    provided with the software.

    This copyright notice may not be removed or altered.
*/


#ifndef __mfcWinBspWaveWrapper__
#define __mfcWinBspWaveWrapper__

#include <afxwin.h>
#include <afxtoolbarbutton.h>

#include "af_return_codes.h"
#include "bspWaveWrapper.h"

class mfcWinBspWaveWrapperImpl : public CWnd {

    CMFCToolBarButton m_styleButton;

public:

    mfcWinBspWaveWrapperImpl(CWnd *parent = 0, int style_flags = 0);
    mfcWinBspWaveWrapperImpl(HWND parent, int style_flags = 0);

    void createBspWaveWrapper(CWnd *parent, int style_flags);
    void createBspWaveWrapper(HWND parent, int style_flags);

    af_return_code executeInteractiveCommandStd(const std::string &cmd);

    af_return_code processCommandLineStd(const std::string &cmd);
    af_return_code processCommandLine(LPSTR cmd);

    af_return_code cmdStd(const std::string &cmd);
    af_return_code cmdClose(bool flag_force);
    const char **getCommandResult();
    void setGeometry(int x, int y, int w, int h) {
        MoveWindow(x, y, w, h);
    }


    virtual void onCloseButtonActivated() {}
    virtual void onAttachButtonActivated() {}
    virtual void onDetachButtonActivated() {}
    virtual void onCurveHighlighted(const std::string &section_name, const std::string &curve_name, int section_id, int curve_index, int flag_clear, int from_browser) {}
    virtual void onCurveUnHighlighted(const std::string &section_name, const std::string &curve_name, int section_id, int curve_index, int flag_clear, int from_browser) {}
    virtual void onCurveActivated(const std::string &section_name, const std::string &curve_name, int section_id, int curve_index, int flag_clear, int from_browser) {}
    virtual void onLoggerMessage(int severity, const std::string &message) {}
    virtual void onStatusBarMessage(int severity, const std::string &message) {}
    virtual void onCheckIntegratorLicence(void *data) {}
    virtual int onNotify(const struct _af_bspwave_callback_notification_type *description);
    virtual void onProgressMessage(const std::string &message, int progress) {}
    virtual void onFileWasOpened(const std::string &message) {}
    virtual void onFileWasClosed(const std::string &message) {}
    virtual void onAllFilesWereClosed() {}
    virtual void onSectionUsedForLinkChanged() {}
    virtual int onOpenFileButtonActivated() { return 0; }

    void registerCallbackFunctions(bool flag_drag_string_gene, bool flag_cursor_values);
    void unregisterCallbackFunctions();

    HWND getBrowserHandle();
    HWND getToolbarHandle();
    HWND getTabCtrlHandle();
    HWND getMeasurementsHandle();
    HWND getPageParametersHandle();
    HWND getPageLayoutHandle();
    HWND getSidebarHandle();

    void getBestSizeForToolbar(int &w, int &h);
    HMENU getMenubarHandle();
    bool checkIfMenuItemIsEnabled(UINT id);
    void executeMenuCommand(UINT id);
    af_return_code registerWidgetCallbackFunctions();
    af_return_code unregisterWidgetCallbackFunctions();

    virtual void onTabCtrlPageAdded(const char *page_name, HWND page_handle) {}
    virtual void onTabCtrlPageRemoved(const char *page_name, HWND page_handle) {}
    virtual void onTabCtrlPageChanged(const char *page_name, HWND page_handle) {}
    virtual void onEnsurePageVisible(const char *page_name, HWND page_handle) {}

    DROPEFFECT handleDragEnter(HWND page_handle, COleDataObject* pDataObject, DWORD dwKeyState);
    BOOL handleDrop(HWND page_handle, COleDataObject* pDataObject, DROPEFFECT dropEffect);
    DROPEFFECT handleDragEnter(HWND page_handle, LPDATAOBJECT pDataObject, DWORD dwKeyState);
    BOOL handleDrop(HWND page_handle, LPDATAOBJECT pDataObject, DROPEFFECT dropEffect);
    void handleDragLeave(HWND page_handle);

    af_return_code registerWaveformParserPlugin(struct _af_wp_plugin_functions_type *wp_plugin_functions);
    af_return_code unregisterWaveformParserPlugin(struct _af_wp_plugin_functions_type *wp_plugin_functions);

    af_return_code registerStyleCallbackFunctions();
    af_return_code unregisterStyleCallbackFunctions();

    int drawToolbarButton(HDC dc, int image_id, int x, int y, int w, int h, int button_status, int *image_was_drawn, int *used_bg_color);

private:

    HWND getHandle(int m_info_type);

};

class mfcWinBspWaveWrapper : public bspWaveWrapper<mfcWinBspWaveWrapperImpl> {

public:

    mfcWinBspWaveWrapper() { }
    mfcWinBspWaveWrapper(CWnd *parent, int style_flags)
        : bspWaveWrapper<mfcWinBspWaveWrapperImpl>(parent, style_flags) {
        }
    mfcWinBspWaveWrapper(HWND parent, int style_flags)
        : bspWaveWrapper<mfcWinBspWaveWrapperImpl>(parent, style_flags) {
        }

};


#endif //__mfcWinBspWaveTest__
