/*
    Copyright Analog Flavor 2013-2024

    All rights reserved.

    This software contains proprietary and confidential information which is the
    property of Analog Flavor and is subject to license terms.

    This file may be used in accordance with the license agreement 
    provided with the software.

    This copyright notice may not be removed or altered.
*/

#ifndef __qtWinBspWaveWrapper__
#define __qtWinBspWaveWrapper__

#include <QWidget>
#include <QResizeEvent>
#include "winBspWaveWrapper.h"

class qtWinBspWaveComponentWrapper : public QWidget {

    WId m_parentId;
    HWND m_oldComponentParent;
    HWND m_bspWaveComponent;
    bool m_flagBlockEvents;

    HWND m_bspWaveMainWindow;

public:

    qtWinBspWaveComponentWrapper(QWidget *parent, HWND component, HWND main_win);
    ~qtWinBspWaveComponentWrapper();

    void showEvent(QShowEvent * event);
    void resizeEvent(QResizeEvent * event);
    bool event(QEvent *event);

    void showComponent();
    void hideComponent();

protected:

    void setBspWaveComponent(HWND w);
    void setParentId(WId parent_id) {
        m_parentId = parent_id;
    }
    virtual void create() { reparent();  }
    void reparent();

private:

    void handleNewSize(int w, int h);
    void handleNewSize();

};

class qtWinBspWaveWrapperImpl : public qtWinBspWaveComponentWrapper, public winBspWaveWrapper {

    int m_style;

public:

    qtWinBspWaveWrapperImpl();
    qtWinBspWaveWrapperImpl(QWidget *parent, int style);

    af_return_code cmdQt(const QString &cmd);
    af_return_code cmdOpenFilesQt(const QStringList &files);

    virtual void create();

protected:


private:

};

class qtWinBspWaveWrapper : public bspWaveWrapper<qtWinBspWaveWrapperImpl> {

public:

    qtWinBspWaveWrapper(QWidget *parent, int style)
        : bspWaveWrapper<qtWinBspWaveWrapperImpl>(parent, style) {
    }


};

#endif //__qtWinBspWaveTest__
