/*
    Copyright Analog Flavor 2013-2024

    All rights reserved.

    This software contains proprietary and confidential information which is the
    property of Analog Flavor and is subject to license terms.

    This file may be used in accordance with the license agreement 
    provided with the software.

    This copyright notice may not be removed or altered.
*/


#ifndef __winBspWaveWrapper__
#define __winBspWaveWrapper__

#include "af_return_codes.h"
#include "bspWaveWrapper.h"
#include "bspWaveWrapperCallbacks.h"

#include <windows.h>

#include <string>
#include <list>

class winBspWaveWrapperImpl : public bspWaveWrapperCallbacks {

    HWND m_bspwaveWidget;
    bool m_flagDestroyObjectWhenClosing;

    std::string m_tmpString;

public:

    winBspWaveWrapperImpl();
    winBspWaveWrapperImpl(HWND parent, int style_flags = 0);
    winBspWaveWrapperImpl(HWND parent, int x, int y, int w, int h, int style_flags);
    virtual ~winBspWaveWrapperImpl() { }

    bool createWidget(HWND parent, int style_flags);
    bool createWidget(HWND parent, int x, int y, int w, int h, int style_flags);
    bool createMainWindow(int style_flags);

    HWND getBspwaveWidgetPtr() { return m_bspwaveWidget; }
    void setBspwaveWidgetPtr(HWND win) { m_bspwaveWidget = win; }

    af_return_code getWaveformParserAccessFunctions(struct _af_wp_access_functions_type *access_functions);

    std::string &tmpString() { return m_tmpString; }
    void setFlagDestroyObjectWhenClosing(bool f) {
        m_flagDestroyObjectWhenClosing = f;
    }

    void setGeometry(int x, int y, int w, int h);
    void hideBspwave();
    void showBspwave();

    af_return_code cmdStd(const std::string &cmd);
    af_return_code cmdClose(bool flag_force);

    af_return_code processCommandLine(LPSTR cmd);

    const char **getCommandResult();

    virtual void onWidgetDestroyed(bool &flag_delete_object) {
        flag_delete_object = m_flagDestroyObjectWhenClosing;
        m_bspwaveWidget = 0;
    }

    void registerCallbackFunctions(bool flag_drag_string_gene, bool flag_cursor_values);
    void unregisterCallbackFunctions();

    af_return_code registerWidgetCallbackFunctions();
    af_return_code unregisterWidgetCallbackFunctions();

    virtual void onTabCtrlPageAdded(const char *page_name, HWND page_handle) {}
    virtual void onTabCtrlPageRemoved(const char *page_name, HWND page_handle) {}
    virtual void onTabCtrlPageChanged(const char *page_name, HWND page_handle) {}
    virtual void onEnsurePageVisible(const char *page_name, HWND page_handle) {}

    af_return_code registerWaveformParserPlugin(struct _af_wp_plugin_functions_type *wp_plugin_functions);
    af_return_code unregisterWaveformParserPlugin(struct _af_wp_plugin_functions_type *wp_plugin_functions);

    HWND getBrowserHandle();
    HWND getMeasurementsHandle();
    HWND getPageParametersHandle();
    HWND getPageLayoutHandle();
    HWND getSidebarHandle();
    HWND getToolbarHandle();
    HWND getTabCtrlHandle();

private:

    HWND getHandle(int m_info_type);

};

class winBspWaveWrapper : public bspWaveWrapper<winBspWaveWrapperImpl> {

public:

    winBspWaveWrapper()
        : bspWaveWrapper<winBspWaveWrapperImpl>() {
    }
    winBspWaveWrapper(HWND parent, int style_flags = 0)
        : bspWaveWrapper<winBspWaveWrapperImpl>(parent, style_flags) {
    }
    winBspWaveWrapper(HWND parent, int x, int y, int w, int h, int style_flags)
        : bspWaveWrapper<winBspWaveWrapperImpl>(parent, x, y, w, h, style_flags) {
    }


};

#endif //__winWinBspWaveTest__
