#
#    Copyright Analog Flavor 2013-2024
#
#    All rights reserved.
#
#    This software contains proprietary and confidential information which is the
#    property of Analog Flavor and is subject to license terms.
#
#    This file may be used in accordance with the license agreement 
#    provided with the software.
#
#    This copyright notice may not be removed or altered.
#
#!/usr/bin/python


import bspwave_win_interface

import PyQt4.QtGui
import PyQt4.QtCore

class QWinBspWaveWidget(PyQt4.QtGui.QWidget, bspwave_win_interface.bspwave_win_interface) :

    m_ParentWindowIdentifier = 0
    m_x = 0
    m_y = 0
    m_width = 1
    m_height = 1

    def __init__(self, parent = None, flag_debug = 0):
        PyQt4.QtGui.QWidget.__init__(self, parent)
        bspwave_win_interface.bspwave_win_interface.__init__(self, flag_debug)

    def computeWidgetPosition(self):
        self.m_x = 0
        self.m_y = 0
        self.m_width = self.width()
        self.m_height = self.height()
        if self.m_ParentWindowIdentifier != 0 :
            native_parent = self.find(self.m_ParentWindowIdentifier)
            if native_parent != 0 :
                p = PyQt4.QtCore.QPoint(0, 0)
                p = self.mapToGlobal(p);
                self.m_x = p.x();
                self.m_y = p.y();
                p = PyQt4.QtCore.QPoint(0, 0)
                p = native_parent.mapToGlobal(p);
                self.m_x = self.m_x - p.x();
                self.m_y = self.m_y - p.y();

    def handleChangingParent(self):
        if self.m_ParentWindowIdentifier != 0 :
            self.winId()
            new_id = self.effectiveWinId()
            if new_id != None:
                if new_id != self.m_ParentWindowIdentifier :
                    self.m_ParentWindowIdentifier = new_id
                    self.reparentBspwaveWidget(self.m_ParentWindowIdentifier)


    def showEvent(self, show_event):
        if self.getWindowIdentifier() == 0 :
            # the bspwave widget must be created
            self.winId()
            self.m_ParentWindowIdentifier = self.effectiveWinId()
            if self.m_ParentWindowIdentifier == None :
                self.m_ParentWindowIdentifier = 0
            if self.m_debug :
                msg = "The parent HWND is {0} ".format(self.m_ParentWindowIdentifier)
                print(msg)
            if self.m_ParentWindowIdentifier != 0 :
                self.computeWidgetPosition()
                self.createBspwaveWidget(self.m_ParentWindowIdentifier, self.m_x, self.m_y, self.m_width, self.m_height, 6)

    def resizeEvent(self, size_event):
        #msg = "resizing widget id == {0} ".format(self.m_ParentWindowIdentifier)
        #print(msg)
        if self.m_ParentWindowIdentifier != 0 :
            self.computeWidgetPosition()
            #msg = "Moving widget to {0} {1} {2} {3}".format(self.m_x, self.m_y, self.m_width, self.m_height)
            #print(msg)
            self.moveBspwaveWidget(self.m_x, self.m_y, self.m_width, self.m_height)

    def event(self, any_event):
        if any_event.type() == PyQt4.QtCore.QEvent.WinIdChange :
            self.handleChangingParent()
        if any_event.type() == PyQt4.QtCore.QEvent.ParentChange :
            self.handleChangingParent()
        return PyQt4.QtGui.QWidget.event(self, any_event);



class QWinBspWaveDockWidget(PyQt4.QtGui.QDockWidget) :

    m_bspwave = 0

    def __init__(self, parent = None, flag_debug = 0):
        PyQt4.QtGui.QDockWidget.__init__(self, "BeSpice Wave", parent)
        self.m_bspwave = QWinBspWaveWidget(self, flag_debug)

        self.m_bspwave.setMinimumSize(200, 200)
        self.m_bspwave.show()

        self.setWidget(self.m_bspwave);

        self.setFeatures(PyQt4.QtGui.QDockWidget.DockWidgetMovable | PyQt4.QtGui.QDockWidget.DockWidgetFloatable)

    def executeInteractiveCommand(self, command):
        return self.m_bspwave.executeInteractiveCommand(command)

    def executeInteractiveCommandOpenFile(self, file_name):
        return self.m_bspwave.executeInteractiveCommandOpenFile(file_name)




