#
#    Copyright Analog Flavor 2013-2024
#
#    All rights reserved.
#
#    This software contains proprietary and confidential information which is the
#    property of Analog Flavor and is subject to license terms.
#
#    This file may be used in accordance with the license agreement 
#    provided with the software.
#
#    This copyright notice may not be removed or altered.
#
#!/usr/bin/python


import os
import sys
import ctypes

class bspwave_win_interface :

    m_libraryName = ""
    m_fileName = ""
    m_libraryHandle = 0
    m_windowIdentifier = 0
    m_debug = 0

    af_code_ok = 0
    af_code_error = 1
    af_code_license_expired = 4
    af_code_bad_name = 6
    af_code_bad_index = 8
    af_code_queued = 11

    def __init__(self, debug = 0):
        self.m_windowIdentifier = 0
        self.m_debug = debug
        if bspwave_win_interface.m_libraryHandle == 0 :
            if bspwave_win_interface.m_fileName == "" :
                self.m_libraryName = "libaf_bspwave_widget_win"
                self.findBspwaveWidgetDll()
            if bspwave_win_interface.m_fileName == "" :
                self.m_libraryName = "af_bspwave_widget_win"
                self.findBspwaveWidgetDll()
            if bspwave_win_interface.m_fileName == "" :
                if self.m_debug :
                    msg = "The library \"{0}\" was not found".format(self.m_libraryName)
                    print(msg)
                    msg = "    Please extend your PATH variable to include \"{0}\".".format(self.m_libraryName)
                    print(msg)
                    msg = "    For example : PATH=%cd%\..\..\..\platform\windows\X86;%PATH%"
                    print(msg)
            else:
                if self.m_debug :
                    msg = "The library was found at \"{0}\"".format(bspwave_win_interface.m_fileName)
                    print(msg)
                bspwave_win_interface.m_libraryHandle = ctypes.cdll.LoadLibrary(bspwave_win_interface.m_fileName)
                if bspwave_win_interface.m_libraryHandle != 0 :
                    ## define the arguments used by the c functions according to the c headers
                    bspwave_win_interface.m_libraryHandle.win_bspwave_create_and_run_main.argtypes = [ctypes.c_int, ctypes.c_int, ctypes.c_int, ctypes.c_int, ctypes.c_char_p]
                    bspwave_win_interface.m_libraryHandle.win_bspwave_create_and_run_main.restype = ctypes.c_int
                    bspwave_win_interface.m_libraryHandle.win_bspwave_execute_command.argtypes = [ctypes.c_int, ctypes.c_char_p]
                    bspwave_win_interface.m_libraryHandle.win_bspwave_execute_command.restype = ctypes.c_int
                    bspwave_win_interface.m_libraryHandle.win_bspwave_get_command_result_line.argtypes = [ctypes.c_int, ctypes.c_int]
                    bspwave_win_interface.m_libraryHandle.win_bspwave_get_command_result_line.restype = ctypes.c_char_p
                    bspwave_win_interface.m_libraryHandle.win_bspwave_create_widget.argtypes = [ctypes.c_int, ctypes.c_int, ctypes.c_int, ctypes.c_int, ctypes.c_int, ctypes.c_int]
                    bspwave_win_interface.m_libraryHandle.win_bspwave_create_widget.restype = ctypes.c_int
                    bspwave_win_interface.m_libraryHandle.win_bspwave_move_widget.argtypes = [ctypes.c_int, ctypes.c_int, ctypes.c_int, ctypes.c_int, ctypes.c_int]
                    bspwave_win_interface.m_libraryHandle.win_bspwave_move_widget.restype = ctypes.c_int
                    bspwave_win_interface.m_libraryHandle.win_bspwave_reparent_widget.argtypes = [ctypes.c_int, ctypes.c_int]
                    bspwave_win_interface.m_libraryHandle.win_bspwave_reparent_widget.restype = ctypes.c_int


    def findSystemPathForFile(self, file_name):
        found_system_path = ""
        system_path = ""
        path_list = []
        if sys.platform.startswith('win'):
            system_path = os.getenv("PATH")
            if isinstance(system_path, str) :
                path_list = system_path.split(";")
        elif sys.platform.startswith('linux'):
            system_path = os.getenv("LD_LIBRARY_PATH")
            if isinstance(system_path, str) :
                path_list = system_path.split(":")
        #msg = "Trying to find module file \"{0}\" in sytem path ( \"{1}\" )".format(file_name, system_path)
        #print(msg)
        for path_item in path_list :
            path_item = path_item.replace('"', '')
            complete_path_item = os.path.join(path_item, file_name)
            #msg = "complete path item >> \"{0}\"".format(complete_path_item)
            #print(msg)
            if os.path.exists(complete_path_item) :
                found_system_path = complete_path_item
                break
        return found_system_path

    def findBspwaveWidgetDll(self):
        ## complete name for windows / linux
        lib_name = self.m_libraryName
        if sys.platform.startswith('win'):
            lib_name = self.m_libraryName + ".dll"
        elif sys.platform.startswith('linux'):
            lib_name = self.m_libraryName + ".so"
        ## find library
        bspwave_win_interface.m_fileName = self.findSystemPathForFile(lib_name)

    def createBspwaveWidget(self, parent, x, y, width, height, window_style = 6):
        if bspwave_win_interface.m_libraryHandle != 0 :
            self.m_windowIdentifier = 0;
            if self.m_debug :
                window_style = window_style | 1
            self.m_windowIdentifier = bspwave_win_interface.m_libraryHandle.win_bspwave_create_widget(parent, x, y, width, height, window_style)

    def moveBspwaveWidget(self, x, y, width, height):
        if self.m_windowIdentifier != 0 :
            #msg = "Moving widget to {0} {1} {2} {3}".format(x, y, width, height)
            #print(msg)
            bspwave_win_interface.m_libraryHandle.win_bspwave_move_widget(self.m_windowIdentifier, x, y, width, height)

    def reparentBspwaveWidget(self, new_parent):
        if self.m_windowIdentifier != 0 :
            bspwave_win_interface.m_libraryHandle.win_bspwave_reparent_widget(self.m_windowIdentifier,new_parent)

    def runBspwaveMain(self, window_width = -1, window_height = -1, window_style = 38):
        if bspwave_win_interface.m_libraryHandle != 0 :
            self.m_windowIdentifier = 0;
            if self.m_debug :
                window_style = window_style | 1
            bspwave_startup_command = ""
            cmd = bspwave_startup_command.encode()
            bspwave_win_interface.m_libraryHandle.win_bspwave_create_and_run_main(window_width, window_height, window_style, 0, cmd)

    def runBspwaveMainInThread(self, window_width = -1, window_height = -1, window_style = 38):
        if bspwave_win_interface.m_libraryHandle != 0 :
            self.m_windowIdentifier = 0
            if self.m_debug :
                window_style = window_style | 1
            bspwave_startup_command = ""
            cmd = bspwave_startup_command.encode()
            self.m_windowIdentifier = bspwave_win_interface.m_libraryHandle.win_bspwave_create_and_run_main(window_width, window_height, window_style, 1, cmd)

    def executeInteractiveCommand(self, command):
        if self.m_windowIdentifier != 0 :
            cmd = command.encode()
            rc = bspwave_win_interface.m_libraryHandle.win_bspwave_execute_command(self.m_windowIdentifier, cmd)
            if self.m_debug :
                if rc == self.af_code_bad_index :
                    msg = "the window identifier {0} is invalid".format(self.m_windowIdentifier)
                    print(msg)
                if rc == self.af_code_bad_name :
                    print("the command is empty")
                if rc == self.af_code_license_expired :
                    print("your license expired")
                if rc == self.af_code_queued :
                    print("the command was queued (it is executed in a thread)")
                #if rc == self.af_code_ok :
                #    print("the command succeeded")
            if rc == self.af_code_ok :
                line = 0
                result = []
                while line < 1000 :
                    result_line = bspwave_win_interface.m_libraryHandle.win_bspwave_get_command_result_line(self.m_windowIdentifier, line)
                    msg = "received line {0}".format(result_line)
                    print(msg)
                    if result_line == "" or result_line == None :
                        break
                    result.append(result_line)
                return result
        return False

    def executeInteractiveCommandOpenFile(self, file_name):
        command = "open_file \"{0}\"".format(file_name)
        return self.executeInteractiveCommand(command)

    def getWindowIdentifier(self):
        #msg = "window id == {0}".format(self.m_windowIdentifier)
        #print(msg)
        return self.m_windowIdentifier;
