#
#    Copyright Analog Flavor 2013-2024
#
#    All rights reserved.
#
#    This software contains proprietary and confidential information which is the
#    property of Analog Flavor and is subject to license terms.
#
#    This file may be used in accordance with the license agreement 
#    provided with the software.
#
#    This copyright notice may not be removed or altered.
#
#!/usr/bin/python

## To run this example from command line, extend your path
##      to include the analog flavor libraries :
## PATH=%cd%\..\..\..\platform\windows\32;%PATH%
## or PATH=%cd%\..\..\..\platform\windows\64;%PATH%
##
## cd /D ...\analog_flavor_msw\examples\bspwave_widget\python_win_test
##
## Run Python
## python.exe example_2_wxpython_bspwave_widget.py

import bspwave_wx_interface
import time
import wx
import sys

class BspWaveExampleFrame(wx.Frame):

    m_bspwaveWrapper = 0
    m_fileOpen = 1111
    m_showControlPanel = 1112
    m_showCommandLogger = 1113
    m_hidePanels = 1114

    def __init__(self, title, flag_debug):
        wx.Frame.__init__(self, None, title=title, pos=(150,150), size=(750,600))
        self.Bind(wx.EVT_CLOSE, self.OnClose)
        panel = wx.Panel(self)

        # Creating the menubar.
        menuBar = wx.MenuBar()

        # Setting up the menu.
        file_menu = wx.Menu()
        file_menu.Append(self.m_fileOpen, "Open File")
        menuBar.Append(file_menu,"File")

        dev_menu = wx.Menu()
        dev_menu.Append(self.m_showControlPanel, "Show Control Panel")
        dev_menu.Append(self.m_showCommandLogger, "Show Command Logger")
        dev_menu.Append(self.m_hidePanels, "Hide Panels")
        menuBar.Append(dev_menu,"Dev")

        self.SetMenuBar(menuBar)

        self.m_bspwaveWrapper = bspwave_wx_interface.wxWinBspWaveWidget(panel, flag_debug)
        box = wx.BoxSizer(wx.VERTICAL)
        box.Add(self.m_bspwaveWrapper, 1, wx.EXPAND | wx.ALL, 10)
        panel.SetSizer(box)
        panel.Layout()

        self.Bind(wx.EVT_MENU, self.OnMenu)

    def OnClose(self, event):
        self.Destroy()

    def OnMenu(self, event):
        if event.GetId() == self.m_fileOpen :
            print("open file .. ")
            dlg = wx.FileDialog(self, "Open Waveform File", "", "", "*.*", wx.OPEN)
            if dlg.ShowModal() == wx.ID_OK:
                file_name = dlg.GetPath()
                if self.m_bspwaveWrapper != 0 :
                    self.m_bspwaveWrapper.executeInteractiveCommandOpenFile(file_name)
        if event.GetId() == self.m_showControlPanel :
            self.m_bspwaveWrapper.executeInteractiveCommand("show_control_panel 1")
        if event.GetId() == self.m_showCommandLogger :
            self.m_bspwaveWrapper.executeInteractiveCommand("show_command_logger 1")
        if event.GetId() == self.m_hidePanels :
            self.m_bspwaveWrapper.executeInteractiveCommand("show_control_panel 0")
            self.m_bspwaveWrapper.executeInteractiveCommand("show_command_logger 0")

app = wx.App(redirect=True)
top = BspWaveExampleFrame("BeSpice Wave wxPython Example", True)
top.Show()
app.MainLoop()



