#
#    Copyright Analog Flavor 2013-2024
#
#    All rights reserved.
#
#    This software contains proprietary and confidential information which is the
#    property of Analog Flavor and is subject to license terms.
#
#    This file may be used in accordance with the license agreement 
#    provided with the software.
#
#    This copyright notice may not be removed or altered.
#
#!/usr/bin/python

## To run this example from command line, extend your path
##      to include the analog flavor libraries :
## PATH=%cd%\..\..\..\platform\windows\32;%PATH%
## or PATH=%cd%\..\..\..\platform\windows\64;%PATH%
##
## cd /D ...\analog_flavor_msw\examples\bspwave_widget\python_win_test
##
## Run Python
## python.exe example_4_pyqt_bspwave_dock_widget.py

import bspwave_qt_interface
import time

from PyQt4 import QtGui, QtCore
import sys


class QWinBspWaveTestMain(QtGui.QMainWindow):

  #  m_bspwave = 0

    def __init__(self, parent = None, flag_debug = 0 ):
        QtGui.QMainWindow.__init__(self, parent)
        self.setWindowTitle('BspWave Windows Widget in QT Docked Widget')

        ## create a main window menu
        menubar = self.menuBar()

        file_menu = menubar.addMenu('&File')
        self.m_openFileAction = QtGui.QAction('&Open File', self)
        file_menu.addAction(self.m_openFileAction)

        dev_menu = menubar.addMenu('&Dev')
        self.m_showControlPanelAction = QtGui.QAction('&Show Control Panel', self)
        dev_menu.addAction(self.m_showControlPanelAction)
        self.m_showCommandLoggerAction = QtGui.QAction('&Show Command Logger', self)
        dev_menu.addAction(self.m_showCommandLoggerAction)
        self.m_hidePanelsAction = QtGui.QAction('&Hide Panels', self)
        dev_menu.addAction(self.m_hidePanelsAction)

        ## connect actions
        self.m_openFileAction.triggered.connect(self.onOpenFile)
        self.m_showControlPanelAction.triggered.connect(self.onShowControlPanel)
        self.m_showCommandLoggerAction.triggered.connect(self.onShowCommandLogger)
        self.m_hidePanelsAction.triggered.connect(self.onHidePanels)

        ## this creates and adds a BeSpice Wave docked widget
        self.m_bspwave = bspwave_qt_interface.QWinBspWaveDockWidget(self, flag_debug)
        self.addDockWidget(QtCore.Qt.LeftDockWidgetArea, self.m_bspwave)

        ## a tab widget with 2 pages is used as centralwidget widget
        tab_widget = QtGui.QTabWidget(self)
        self.setCentralWidget(tab_widget)
        tab_1 = QtGui.QTextEdit(tab_widget);
        tab_widget.addTab(tab_1, "Tab 1")
        tab_2 = QtGui.QTextEdit(tab_widget);
        tab_widget.addTab(tab_2, "Tab 2")

        ## this creates and adds another docked widget
        dock_2 = QtGui.QDockWidget("Dock 2", self)
        dock_2_contents = QtGui.QTextEdit(dock_2);
        dock_2.setWidget(dock_2_contents)
        self.addDockWidget(QtCore.Qt.LeftDockWidgetArea, dock_2)

    def onOpenFile(self):
        #print "open file .. "
        file_name = QtGui.QFileDialog.getOpenFileName(self, 'Open Waveform File', '.')
        self.m_bspwave.executeInteractiveCommandOpenFile(file_name)

    def onShowControlPanel(self):
        #print "show control panel .. "
        self.m_bspwave.executeInteractiveCommand("show_control_panel 1")

    def onShowCommandLogger(self):
        self.m_bspwave.executeInteractiveCommand("show_command_logger 1")

    def onHidePanels(self):
        self.m_bspwave.executeInteractiveCommand("show_control_panel 0")
        self.m_bspwave.executeInteractiveCommand("show_command_logger 0")


app = QtGui.QApplication(sys.argv)
bspw = QWinBspWaveTestMain(None, 1)
bspw.show()
app.exec_()




