/*
    Copyright Analog Flavor 2013-2024

    All rights reserved.

    This software contains proprietary and confidential information which is the
    property of Analog Flavor and is subject to license terms.

    This file may be used in accordance with the license agreement 
    provided with the software.

    This copyright notice may not be removed or altered.
*/


#include "mfcWinBspWaveWrapper.h"

//#include <windows.h>
#include "bspwave_win_widget.h"
#include "af_wp_access_functions.h"
#include "af_win_widget_style.h"
#include "bspWaveWrapper.impl.h"
#include <sstream>

#include <afxole.h>
#include <afxvisualmanager.h>

template class bspWaveWrapper<mfcWinBspWaveWrapperImpl>;


/// callback functions
extern "C" {

    void mfc_bsp_wave_wrapper_curve_highlighted(void *obj, const char *section_name, const char *curve_name, int section_id, int curve_index, int flag_clear, int from_browser) {
        if ( obj && section_name && curve_name ) {
            mfcWinBspWaveWrapperImpl *wrapper = (mfcWinBspWaveWrapperImpl*)( obj );
            wrapper->onCurveHighlighted(std::string(section_name), std::string(curve_name), section_id, curve_index, flag_clear, from_browser);
        }
    }
    void mfc_bsp_wave_wrapper_curve_un_highlighted(void *obj, const char *section_name, const char *curve_name, int section_id, int curve_index, int flag_clear, int from_browser) {
        if ( obj && section_name && curve_name ) {
            mfcWinBspWaveWrapperImpl *wrapper = (mfcWinBspWaveWrapperImpl*)( obj );
            wrapper->onCurveUnHighlighted(std::string(section_name), std::string(curve_name), section_id, curve_index, flag_clear, from_browser);
        }
    }
    void mfc_bsp_wave_wrapper_curve_activated(void *obj, const char *section_name, const char *curve_name, int section_id, int curve_index, int flag_clear, int from_browser) {
        if ( obj && section_name && curve_name ) {
            mfcWinBspWaveWrapperImpl *wrapper = (mfcWinBspWaveWrapperImpl*)( obj );
            wrapper->onCurveActivated(std::string(section_name), std::string(curve_name), section_id, curve_index, flag_clear, from_browser);
        }
    }
    void mfc_bsp_wave_wrapper_check_integrator_licence(void *obj, void *data) {
        if ( obj ) {
            mfcWinBspWaveWrapperImpl *wrapper = (mfcWinBspWaveWrapperImpl*)( obj );
            wrapper->onCheckIntegratorLicence(data);
        }
    }
    void mfc_bsp_wave_wrapper_tab_ctrl_page_added(void *obj, const char *page_name, HWND page_handle) {
        if (obj) {
            mfcWinBspWaveWrapperImpl *wrapper = (mfcWinBspWaveWrapperImpl*)(obj);
            wrapper->onTabCtrlPageAdded(page_name, page_handle);
        }
    }
    void mfc_bsp_wave_wrapper_tab_ctrl_page_removed(void *obj, const char *page_name, HWND page_handle) {
        if (obj) {
            mfcWinBspWaveWrapperImpl *wrapper = (mfcWinBspWaveWrapperImpl*)(obj);
            wrapper->onTabCtrlPageRemoved(page_name, page_handle);
        }
    }
    void mfc_bsp_wave_wrapper_tab_ctrl_page_changed(void *obj, const char *page_name, HWND page_handle) {
        if (obj) {
            mfcWinBspWaveWrapperImpl *wrapper = (mfcWinBspWaveWrapperImpl*)(obj);
            wrapper->onTabCtrlPageChanged(page_name, page_handle);
        }
    }
    void mfc_bsp_wave_wrapper_ensure_page_visible(void *obj, const char *page_name, HWND page_handle) {
        if ( obj ) {
            mfcWinBspWaveWrapperImpl *wrapper = (mfcWinBspWaveWrapperImpl*)( obj );
            wrapper->onEnsurePageVisible(page_name, page_handle);
        }
    }
    int mfc_bsp_wave_wrapper_draw_toolbar_button(void *obj, HDC dc, int image_id, int x, int y, int w, int h, int button_status, int *image_was_drawn, int *used_bg_color) {
        if ( obj ) {
            mfcWinBspWaveWrapperImpl *wrapper = (mfcWinBspWaveWrapperImpl*)( obj );
            return wrapper->drawToolbarButton(dc, image_id, x, y, w, h, button_status, image_was_drawn, used_bg_color);
        }
        return 0;
    }
    int mfc_bsp_wave_wrapper_notify(void *obj, const af_bspwave_callback_notification_type *description) {
        if ( obj && description ) {
            mfcWinBspWaveWrapperImpl *wrapper = (mfcWinBspWaveWrapperImpl*)( obj );
            return wrapper->onNotify(description);
        }
        return 0;
    }

} /// extern c


mfcWinBspWaveWrapperImpl::mfcWinBspWaveWrapperImpl(CWnd *parent, int style_flags) {
    createBspWaveWrapper(parent, style_flags);
}

mfcWinBspWaveWrapperImpl::mfcWinBspWaveWrapperImpl(HWND parent, int style_flags) {
    createBspWaveWrapper(parent, style_flags);
}

void mfcWinBspWaveWrapperImpl::createBspWaveWrapper(CWnd *parent, int style_flags) {
    if ( parent ) {
        HWND parent_hwnd = parent->GetSafeHwnd();
        createBspWaveWrapper(parent_hwnd, style_flags);
    }
}

void mfcWinBspWaveWrapperImpl::createBspWaveWrapper(HWND parent, int style_flags) {
    RECT rect;
    ::GetClientRect(parent, &rect);
    HWND current_hwnd = win_bspwave_create_widget(parent, 0, 0, rect.right, rect.bottom, style_flags);
    Attach(current_hwnd);
    registerCallbackFunctions(false, false);
}

af_return_code mfcWinBspWaveWrapperImpl::executeInteractiveCommandStd(const std::string &cmd) {
    af_return_code rc = af_code_error;
    const char *c_cmd = cmd.c_str();
    HWND current_hwnd = GetSafeHwnd();
    if (current_hwnd) {
        rc = win_bspwave_execute_command(current_hwnd, c_cmd);
    }
    return rc;
}

af_return_code mfcWinBspWaveWrapperImpl::processCommandLineStd(const std::string &cmd) {
    af_return_code rc = af_code_error;
    HWND current_hwnd = GetSafeHwnd();
    if (current_hwnd) {
        const char *c_cmd = cmd.c_str();
        rc = win_bspwave_process_command_line(current_hwnd, (LPSTR)c_cmd);
    }
    return rc;
}

af_return_code mfcWinBspWaveWrapperImpl::cmdStd(const std::string &cmd) {
    af_return_code rc = af_code_error;
    HWND current_hwnd = GetSafeHwnd();
    if (current_hwnd) {
        const char *c_cmd = cmd.c_str();
        rc = win_bspwave_execute_command(current_hwnd, c_cmd);
    }
    return rc;
}

af_return_code mfcWinBspWaveWrapperImpl::cmdClose(bool flag_force) {
    af_return_code rc = af_code_error;
    if ( flag_force ) {
        /// files in text editor might not be saved
        rc = cmdStd(std::string("close 1"));
    }
    else {
        rc = cmdStd(std::string("close 0"));
    }
    return rc;
}

const char **mfcWinBspWaveWrapperImpl::getCommandResult() {
    HWND current_hwnd = GetSafeHwnd();
    if (current_hwnd) {
        return win_bspwave_get_command_result(current_hwnd);
    }
    return 0;
}

af_return_code mfcWinBspWaveWrapperImpl::processCommandLine(LPSTR cmd) {
    af_return_code rc = af_code_error;
    HWND current_hwnd = GetSafeHwnd();
    if (current_hwnd) {
        rc = win_bspwave_process_command_line(current_hwnd, cmd);
    }
    return rc;
}

void mfcWinBspWaveWrapperImpl::registerCallbackFunctions(bool flag_drag_string_gene, bool flag_cursor_values) {
    af_bspwave_callback_type callbacks;
    memset(&callbacks, 0, sizeof( af_bspwave_callback_type ));

    callbacks.callback_object = this;
    callbacks.curve_highlighted = mfc_bsp_wave_wrapper_curve_highlighted;
    callbacks.curve_un_highlighted = mfc_bsp_wave_wrapper_curve_un_highlighted;
    callbacks.curve_activated = mfc_bsp_wave_wrapper_curve_activated;
    callbacks.check_integrator_licence = mfc_bsp_wave_wrapper_check_integrator_licence;
    callbacks.notify = mfc_bsp_wave_wrapper_notify;

    HWND current_hwnd = GetSafeHwnd();
    if ( current_hwnd ) {
        win_bspwave_register_callback(current_hwnd, &callbacks);
    }
}
void mfcWinBspWaveWrapperImpl::unregisterCallbackFunctions() {
    af_bspwave_callback_type callbacks;
    memset(&callbacks, 0, sizeof( af_bspwave_callback_type ));
    HWND current_hwnd = GetSafeHwnd();
    if ( current_hwnd ) {
        win_bspwave_register_callback(current_hwnd, &callbacks);
    }
}

HWND mfcWinBspWaveWrapperImpl::getHandle(int m_info_type) {
    HWND current_hwnd = GetSafeHwnd();
    if (current_hwnd) {
        struct af_bspwave_win_widget_info_type widget_info;
        memset(&widget_info, 0, sizeof(af_bspwave_win_widget_info_type));
        widget_info.m_info_type = m_info_type;
        win_bspwave_get_widget_info(current_hwnd, &widget_info);
        return widget_info.m_widget_handle;
    }
    return 0;
}

HWND mfcWinBspWaveWrapperImpl::getBrowserHandle() {
    return getHandle(AF_BSPWAVE_WIDGET_INFO_TYPE_BROWSER_HANDLE);
}

HWND mfcWinBspWaveWrapperImpl::getMeasurementsHandle() {
    return getHandle(AF_BSPWAVE_WIDGET_INFO_TYPE_MEASUREMENTS_HANDLE);
}

HWND mfcWinBspWaveWrapperImpl::getPageParametersHandle() {
    return getHandle(AF_BSPWAVE_WIDGET_INFO_TYPE_PAGE_PARAMETERS_HANDLE);
}

HWND mfcWinBspWaveWrapperImpl::getPageLayoutHandle() {
    return getHandle(AF_BSPWAVE_WIDGET_INFO_TYPE_LAYOUT_HANDLE);
}

HWND mfcWinBspWaveWrapperImpl::getSidebarHandle() {
    return getHandle(AF_BSPWAVE_WIDGET_INFO_TYPE_SIDEBAR_HANDLE);
}

HWND mfcWinBspWaveWrapperImpl::getToolbarHandle() {
    return getHandle(AF_BSPWAVE_WIDGET_INFO_TYPE_TOOLBAR_HANDLE);
}
HWND mfcWinBspWaveWrapperImpl::getTabCtrlHandle() {
    return getHandle(AF_BSPWAVE_WIDGET_INFO_TYPE_TAB_CTRL_HANDLE);
}
void mfcWinBspWaveWrapperImpl::getBestSizeForToolbar(int &w, int &h) {
    HWND current_hwnd = GetSafeHwnd();
    if (current_hwnd) {
        struct af_bspwave_win_widget_info_type widget_info;
        memset(&widget_info, 0, sizeof(af_bspwave_win_widget_info_type));
        widget_info.m_info_type = AF_BSPWAVE_WIDGET_INFO_TYPE_TOOLBAR_BEST_SIZE;
        win_bspwave_get_widget_info(current_hwnd, &widget_info);
        w = widget_info.m_width;
        h = widget_info.m_height;
    }
}
HMENU mfcWinBspWaveWrapperImpl::getMenubarHandle() {
    HWND current_hwnd = GetSafeHwnd();
    if (current_hwnd) {
        struct af_bspwave_win_widget_info_type widget_info;
        memset(&widget_info, 0, sizeof(af_bspwave_win_widget_info_type));
        widget_info.m_info_type = AF_BSPWAVE_WIDGET_INFO_TYPE_MENUBAR_HANDLE;
        win_bspwave_get_widget_info(current_hwnd, &widget_info);
        return widget_info.m_menu_handle;
    }
    return 0;
}
bool mfcWinBspWaveWrapperImpl::checkIfMenuItemIsEnabled(UINT id) {
    HWND current_hwnd = GetSafeHwnd();
    if (current_hwnd) {
        struct af_bspwave_win_widget_info_type widget_info;
        memset(&widget_info, 0, sizeof(af_bspwave_win_widget_info_type));
        widget_info.m_info_type = AF_BSPWAVE_WIDGET_INFO_TYPE_CHECK_MENU_ITEM_ENABLED;
        widget_info.m_id = id;
        win_bspwave_get_widget_info(current_hwnd, &widget_info);
        return widget_info.m_flag;
    }
    return false;
}
void mfcWinBspWaveWrapperImpl::executeMenuCommand(UINT id) {
    HWND current_hwnd = GetSafeHwnd();
    if (current_hwnd) {
        struct af_bspwave_win_widget_info_type widget_info;
        memset(&widget_info, 0, sizeof(af_bspwave_win_widget_info_type));
        widget_info.m_info_type = AF_BSPWAVE_WIDGET_INFO_TYPE_EXECUTE_MENU_COMMAND;
        widget_info.m_id = id;
        win_bspwave_get_widget_info(current_hwnd, &widget_info);
    }
}
af_return_code mfcWinBspWaveWrapperImpl::registerWidgetCallbackFunctions() {
    HWND current_hwnd = GetSafeHwnd();
    if (current_hwnd) {
        af_bspwave_win_callback_type callbacks;
        memset(&callbacks, 0, sizeof(af_bspwave_win_callback_type));
        callbacks.callback_object = this;
        callbacks.tab_ctrl_page_added = mfc_bsp_wave_wrapper_tab_ctrl_page_added;
        callbacks.tab_ctrl_page_changed = mfc_bsp_wave_wrapper_tab_ctrl_page_changed;
        callbacks.tab_ctrl_page_removed = mfc_bsp_wave_wrapper_tab_ctrl_page_removed;
        callbacks.ensure_page_visible = mfc_bsp_wave_wrapper_ensure_page_visible;
        return win_bspwave_register_win_callback(current_hwnd, &callbacks);
    }
    return af_code_error;
}
af_return_code mfcWinBspWaveWrapperImpl::unregisterWidgetCallbackFunctions() {
    HWND current_hwnd = GetSafeHwnd();
    if ( current_hwnd ) {
        af_bspwave_win_callback_type callbacks;
        memset(&callbacks, 0, sizeof( af_bspwave_win_callback_type ));
        return win_bspwave_register_win_callback(current_hwnd, &callbacks);
    }
    return af_code_error;
}
DROPEFFECT mfcWinBspWaveWrapperImpl::handleDragEnter(HWND page_handle, COleDataObject* pDataObject, DWORD dwKeyState) {
    if ( pDataObject ) {
        return handleDragEnter(page_handle, pDataObject->m_lpDataObject, dwKeyState);
    }
    return DROPEFFECT_NONE;
}
DROPEFFECT mfcWinBspWaveWrapperImpl::handleDragEnter(HWND page_handle, LPDATAOBJECT pDataObject, DWORD dwKeyState) {
    HWND current_hwnd = GetSafeHwnd();
    if ( current_hwnd ) {
        return win_bspwave_handle_drag_enter(current_hwnd, page_handle, pDataObject, dwKeyState);
    }
    return DROPEFFECT_NONE;
}
BOOL mfcWinBspWaveWrapperImpl::handleDrop(HWND page_handle, COleDataObject* pDataObject, DROPEFFECT dropEffect) {
    if ( pDataObject ) {
        return handleDrop(page_handle, pDataObject->m_lpDataObject, dropEffect);
    }
    return FALSE;
}
BOOL mfcWinBspWaveWrapperImpl::handleDrop(HWND page_handle, LPDATAOBJECT pDataObject, DROPEFFECT dropEffect) {
    HWND current_hwnd = GetSafeHwnd();
    if ( current_hwnd ) {
        return win_bspwave_handle_drop(current_hwnd, page_handle, pDataObject, dropEffect);
    }
    return FALSE;
}
void mfcWinBspWaveWrapperImpl::handleDragLeave(HWND page_handle) {
    HWND current_hwnd = GetSafeHwnd();
    if ( current_hwnd ) {
        win_bspwave_handle_drag_leave(current_hwnd, page_handle);
    }
}
af_return_code mfcWinBspWaveWrapperImpl::registerWaveformParserPlugin(struct _af_wp_plugin_functions_type *wp_plugin_functions) {
    af_return_code rc = af_code_error;
    HWND current_hwnd = GetSafeHwnd();
    if ( current_hwnd && wp_plugin_functions ) {
        af_bspwave_plugin_waveform_parser_type wp_functions;
        rc = win_bspwave_get_waveform_parser(current_hwnd, &wp_functions);
        if ( rc == af_code_ok ) {
            if ( wp_functions.register_plugin_functions && wp_functions.parser_object ) {
                rc = ( wp_functions.register_plugin_functions )( wp_functions.parser_object, wp_plugin_functions );
            }
        }
    }
    return rc;
}
af_return_code mfcWinBspWaveWrapperImpl::unregisterWaveformParserPlugin(struct _af_wp_plugin_functions_type *wp_plugin_functions) {
    af_return_code rc = af_code_error;
    HWND current_hwnd = GetSafeHwnd();
    if ( current_hwnd && wp_plugin_functions ) {
        af_bspwave_plugin_waveform_parser_type wp_functions;
        rc = win_bspwave_get_waveform_parser(current_hwnd, &wp_functions);
        if ( rc == af_code_ok ) {
            if ( wp_functions.unregister_plugin_functions && wp_functions.parser_object ) {
                rc = ( wp_functions.unregister_plugin_functions )( wp_functions.parser_object, wp_plugin_functions );
            }
        }
    }
    return rc;
}
af_return_code mfcWinBspWaveWrapperImpl::registerStyleCallbackFunctions() {
    af_return_code rc = af_code_error;
    HWND current_hwnd = GetSafeHwnd();
    if ( current_hwnd ) {
        af_win_widget_style_callback_type style_functions;
        memset(&style_functions, 0, sizeof( af_win_widget_style_callback_type ));
        style_functions.m_structure_size = sizeof( af_win_widget_style_callback_type );
        style_functions.m_callback_object = this;
        style_functions.m_draw_toolbar_button = mfc_bsp_wave_wrapper_draw_toolbar_button;
        rc = win_bspwave_register_style_callback(current_hwnd, &style_functions);
    }
    return rc;
}
af_return_code mfcWinBspWaveWrapperImpl::unregisterStyleCallbackFunctions() {
    af_return_code rc = af_code_error;
    HWND current_hwnd = GetSafeHwnd();
    if ( current_hwnd ) {
        af_win_widget_style_callback_type style_functions;
        memset(&style_functions, 0, sizeof( af_win_widget_style_callback_type ));
        style_functions.m_structure_size = sizeof( af_win_widget_style_callback_type );
        rc = win_bspwave_register_style_callback(current_hwnd, &style_functions);
    }
    return rc;
}

int mfcWinBspWaveWrapperImpl::drawToolbarButton(HDC dc, int image_id, int x, int y, int w, int h, int button_status, int *image_was_drawn, int *used_bg_color) {
    CMFCVisualManager *mgr = CMFCVisualManager::GetInstance();
    if ( mgr ) {
        CDC pDC;
        pDC.Attach(dc);
        CRect rect(x, y, x + w, y + h);
        CMFCVisualManager::AFX_BUTTON_STATE state = CMFCVisualManager::ButtonsIsRegular;
        m_styleButton.EnableWindow(true);
        switch ( button_status ) {
        case AF_STYLE_BUTTON_STATUS_DISABLED:
            m_styleButton.EnableWindow(false);
            break;
        case AF_STYLE_BUTTON_STATUS_PRESSED:
            state = CMFCVisualManager::ButtonsIsPressed;
            break;
        case AF_STYLE_BUTTON_STATUS_HIGHLIGHTED:
            state = CMFCVisualManager::ButtonsIsHighlighted;
            break;
        default:
            break;
        }
        mgr->OnFillButtonInterior(&pDC, &m_styleButton, rect, state);
        mgr->OnDrawButtonBorder(&pDC, &m_styleButton, rect, state);
        pDC.Detach();
        return 1;
    }
    return 0;
}
int mfcWinBspWaveWrapperImpl::onNotify(const struct _af_bspwave_callback_notification_type *description) {
    std::string message;
    if ( description->m_string_value ) {
        message = std::string(description->m_string_value);
    }
    switch ( description->m_type ) {
    case af_bspwave_notification_logger_message:
        onLoggerMessage(description->m_integer_value, message);
        break;
    case af_bspwave_notification_status_bar_message:
        onStatusBarMessage(description->m_integer_value, message);
        break;
    case af_bspwave_notification_progress_message:
        onProgressMessage(message, description->m_integer_value);
        break;
    case af_bspwave_notification_file_opened:
        onFileWasOpened(message);
        break;
    case af_bspwave_notification_file_closed:
        onFileWasClosed(message);
        break;
    case af_bspwave_notification_all_files_closed:
        onAllFilesWereClosed();
        break;
    case af_bspwave_notification_section_used_for_link_changed:
        onSectionUsedForLinkChanged();
        break;
    case af_bspwave_notification_close_button_activated:
        onCloseButtonActivated();
        break;
    case af_bspwave_notification_attach_button_activated:
        onAttachButtonActivated();
        break;
    case af_bspwave_notification_detach_button_activated:
        onDetachButtonActivated();
        break;
    case af_bspwave_notification_open_file_button_activated:
        return onOpenFileButtonActivated();
    default:
        break;
    }
    return 0;
}
