/*
    Copyright Analog Flavor 2013-2024

    All rights reserved.

    This software contains proprietary and confidential information which is the
    property of Analog Flavor and is subject to license terms.

    This file may be used in accordance with the license agreement 
    provided with the software.

    This copyright notice may not be removed or altered.
*/


#include "winBspWaveWrapper.h"
#include "bspwave_win_widget.h"
#include "af_wp_access_functions.h"

#include "bspWaveWrapper.impl.h"
#include <sstream>

#define UNUSED(x) (void)x

template class bspWaveWrapper<winBspWaveWrapperImpl>;

/// callback functions
extern "C" {

    void win_bsp_wave_wrapper_widget_destroyed(void *obj) {
        bool flag_delete_object = false;
        if ( obj ) {
            winBspWaveWrapperImpl *wrapper = (winBspWaveWrapperImpl*)( obj );
            wrapper->onWidgetDestroyed(flag_delete_object);
            if ( flag_delete_object ) {
                delete wrapper;
            }
        }
    }
    void win_bsp_wave_wrapper_curve_highlighted(void *obj, const char *section_name, const char *curve_name, int section_id, int curve_index, int flag_clear, int from_browser) {
        if ( obj && section_name && curve_name ) {
            winBspWaveWrapperImpl *wrapper = (winBspWaveWrapperImpl*)(obj);
            wrapper->onCurveHighlighted(std::string(section_name), std::string(curve_name), section_id, curve_index, flag_clear, from_browser);
        }
    }
    void win_bsp_wave_wrapper_curve_un_highlighted(void *obj, const char *section_name, const char *curve_name, int section_id, int curve_index, int flag_clear, int from_browser) {
        if ( obj && section_name && curve_name ) {
            winBspWaveWrapperImpl *wrapper = (winBspWaveWrapperImpl*)(obj);
            wrapper->onCurveUnHighlighted(std::string(section_name), std::string(curve_name), section_id, curve_index, flag_clear, from_browser);
        }
    }
    void win_bsp_wave_wrapper_curve_activated(void *obj, const char *section_name, const char *curve_name, int section_id, int curve_index, int flag_clear, int from_browser) {
        if ( obj && section_name && curve_name ) {
            winBspWaveWrapperImpl *wrapper = (winBspWaveWrapperImpl*)(obj);
            wrapper->onCurveActivated(std::string(section_name), std::string(curve_name), section_id, curve_index, flag_clear, from_browser);
        }
    }
    const char *win_bsp_wave_wrapper_drop_received(void *obj, const char *plot_name, const char *received_text) {
        if ( obj && plot_name && received_text ) {
            winBspWaveWrapperImpl *wrapper = (winBspWaveWrapperImpl*)( obj );
            wrapper->tmpString() = std::string(received_text);
            if ( wrapper->onDropReceived(std::string(plot_name), wrapper->tmpString()) ) {
                return wrapper->tmpString().c_str();
            }
        }
        return 0;
    }
    const char *win_bsp_wave_wrapper_generate_drag_string(void *obj, const char *xml_drag_string) {
        if ( obj && xml_drag_string ) {
            winBspWaveWrapperImpl *wrapper = (winBspWaveWrapperImpl*)( obj );
            std::string xml_string(xml_drag_string);
            if ( wrapper->onGenerateDragString(xml_string, wrapper->tmpString()) ) {
                return wrapper->tmpString().c_str();
            }
        }
        return 0;
    }
    void win_bsp_wave_wrapper_check_integrator_licence(void *obj, void *data) {
        if ( obj ) {
            winBspWaveWrapperImpl *wrapper = (winBspWaveWrapperImpl*)(obj);
            wrapper->onCheckIntegratorLicence(data);
        }
    }
    void win_bsp_wave_wrapper_tab_ctrl_page_added(void *obj, const char *page_name, HWND page_handle) {
        if ( obj ) {
            winBspWaveWrapperImpl *wrapper = (winBspWaveWrapperImpl*)( obj );
            wrapper->onTabCtrlPageAdded(page_name, page_handle);
        }
    }
    void win_bsp_wave_wrapper_tab_ctrl_page_removed(void *obj, const char *page_name, HWND page_handle) {
        if ( obj ) {
            winBspWaveWrapperImpl *wrapper = (winBspWaveWrapperImpl*)( obj );
            wrapper->onTabCtrlPageRemoved(page_name, page_handle);
        }
    }
    void win_bsp_wave_wrapper_tab_ctrl_page_changed(void *obj, const char *page_name, HWND page_handle) {
        if ( obj ) {
            winBspWaveWrapperImpl *wrapper = (winBspWaveWrapperImpl*)( obj );
            wrapper->onTabCtrlPageChanged(page_name, page_handle);
        }
    }
    void win_bsp_wave_wrapper_ensure_page_visible(void *obj, const char *page_name, HWND page_handle) {
        if ( obj ) {
            winBspWaveWrapperImpl *wrapper = (winBspWaveWrapperImpl*)( obj );
            wrapper->onEnsurePageVisible(page_name, page_handle);
        }
    }
    void win_bsp_wave_wrapper_cursor_values_changed(void *obj, const char *plot_name, const char* const* cursor_values) {
        if ( obj && plot_name && cursor_values ) {
            winBspWaveWrapperImpl *wrapper = (winBspWaveWrapperImpl*)( obj );
            wrapper->onCursorValuesChanged(std::string(plot_name), cursor_values);
        }
    }
    int win_bsp_wave_wrapper_notify(void *obj, const af_bspwave_callback_notification_type *description) {
        if ( obj && description ) {
            winBspWaveWrapperImpl *wrapper = (winBspWaveWrapperImpl*)( obj );
            return wrapper->onNotify(description);
        }
        return 0;
    }

} /// extern c





winBspWaveWrapperImpl::winBspWaveWrapperImpl() {
    m_bspwaveWidget = 0;
    m_flagDestroyObjectWhenClosing = false;
}

winBspWaveWrapperImpl::winBspWaveWrapperImpl(HWND parent, int style_flags) {
    m_bspwaveWidget = 0;
    m_flagDestroyObjectWhenClosing = false;
    createWidget(parent, style_flags);
}
winBspWaveWrapperImpl::winBspWaveWrapperImpl(HWND parent, int x, int y, int w, int h, int style_flags) {
    m_bspwaveWidget = 0;
    m_flagDestroyObjectWhenClosing = false;
    createWidget(parent, x, y, w, h, style_flags);
}
bool winBspWaveWrapperImpl::createWidget(HWND parent, int style_flags) {
    return  createWidget(parent, 0, 0, 100, 100, style_flags);
}
bool winBspWaveWrapperImpl::createWidget(HWND parent, int x, int y, int w, int h, int style_flags) {
    if (m_bspwaveWidget) {
        return false;
    }
    m_bspwaveWidget = win_bspwave_create_widget(parent, x, y, w, h, style_flags);
    if ( m_bspwaveWidget ) {
        registerCallbackFunctions(false, false);
        return true;
    }
    return false;
}
bool winBspWaveWrapperImpl::createMainWindow(int style_flags) {
    if ( m_bspwaveWidget ) {
        return false;
    }
    m_bspwaveWidget = win_bspwave_create_and_run_main(-1, -1, style_flags, true, 0);
    if (m_bspwaveWidget) {
        registerCallbackFunctions(false, false);
        return true;
    }
    return false;
}
void winBspWaveWrapperImpl::setGeometry(int x, int y, int w, int h) {
    if ( m_bspwaveWidget ) {
        win_bspwave_move_widget(m_bspwaveWidget, x, y, w, h);
    }
}
void winBspWaveWrapperImpl::hideBspwave() {
    if ( m_bspwaveWidget ) {
        ::ShowWindow(m_bspwaveWidget, SW_HIDE);
    }
}
void winBspWaveWrapperImpl::showBspwave() {
    if ( m_bspwaveWidget ) {
        ::ShowWindow(m_bspwaveWidget, SW_SHOW);
    }
}
af_return_code winBspWaveWrapperImpl::cmdStd(const std::string &cmd) {
    af_return_code rc = af_code_error;
    if ( m_bspwaveWidget ) {
        const char *c_cmd = cmd.c_str();
        rc = win_bspwave_execute_command(m_bspwaveWidget, c_cmd);
    }
    return rc;
}
af_return_code winBspWaveWrapperImpl::cmdClose(bool flag_force) {
    af_return_code rc = af_code_error;
    if ( flag_force ) {
        /// files in text editor might not be saved
        rc = cmdStd(std::string("close 1"));
    }
    else {
        rc = cmdStd(std::string("close 0"));
    }
    return rc;
}
const char **winBspWaveWrapperImpl::getCommandResult() {
    if ( m_bspwaveWidget ) {
        return win_bspwave_get_command_result(m_bspwaveWidget);
    }
    return 0;
}
af_return_code winBspWaveWrapperImpl::processCommandLine(LPSTR cmd) {
    af_return_code rc = af_code_error;
    if (m_bspwaveWidget) {
        rc = win_bspwave_process_command_line(m_bspwaveWidget, cmd);
    }
    return rc;
}
void winBspWaveWrapperImpl::registerCallbackFunctions(bool flag_drag_string_gene, bool flag_cursor_values) {
    af_bspwave_callback_type callbacks;
    memset(&callbacks, 0, sizeof(af_bspwave_callback_type));

    callbacks.callback_object = this;
    callbacks.widget_destroyed = win_bsp_wave_wrapper_widget_destroyed;
    callbacks.curve_highlighted = win_bsp_wave_wrapper_curve_highlighted;
    callbacks.curve_un_highlighted = win_bsp_wave_wrapper_curve_un_highlighted;
    callbacks.curve_activated = win_bsp_wave_wrapper_curve_activated;
    callbacks.notify = win_bsp_wave_wrapper_notify;
    callbacks.drop_received = win_bsp_wave_wrapper_drop_received;
    if ( flag_drag_string_gene ) {
        callbacks.generate_drag_string = win_bsp_wave_wrapper_generate_drag_string;
    }
    if ( flag_cursor_values ) {
        callbacks.cursor_values_changed = win_bsp_wave_wrapper_cursor_values_changed;
    }
    callbacks.check_integrator_license = win_bsp_wave_wrapper_check_integrator_licence;

    if ( m_bspwaveWidget ) {
        win_bspwave_register_callback(m_bspwaveWidget, &callbacks);
    }
}
void winBspWaveWrapperImpl::unregisterCallbackFunctions() {
    if ( m_bspwaveWidget ) {
        af_bspwave_callback_type callbacks;
        memset(&callbacks, 0, sizeof( af_bspwave_callback_type ));
        win_bspwave_register_callback(m_bspwaveWidget, &callbacks);
    }
}
af_return_code winBspWaveWrapperImpl::registerWidgetCallbackFunctions() {
    if ( m_bspwaveWidget ) {
        af_bspwave_win_callback_type callbacks;
        memset(&callbacks, 0, sizeof( af_bspwave_win_callback_type ));
        callbacks.callback_object = this;
        callbacks.tab_ctrl_page_added = win_bsp_wave_wrapper_tab_ctrl_page_added;
        callbacks.tab_ctrl_page_changed = win_bsp_wave_wrapper_tab_ctrl_page_changed;
        callbacks.tab_ctrl_page_removed = win_bsp_wave_wrapper_tab_ctrl_page_removed;
        callbacks.ensure_page_visible = win_bsp_wave_wrapper_ensure_page_visible;
        return win_bspwave_register_win_callback(m_bspwaveWidget, &callbacks);
    }
    return af_code_error;
}
af_return_code winBspWaveWrapperImpl::unregisterWidgetCallbackFunctions() {
    if ( m_bspwaveWidget ) {
        af_bspwave_win_callback_type callbacks;
        memset(&callbacks, 0, sizeof( af_bspwave_win_callback_type ));
        return win_bspwave_register_win_callback(m_bspwaveWidget, &callbacks);
    }
    return af_code_error;
}
af_return_code winBspWaveWrapperImpl::registerWaveformParserPlugin(struct _af_wp_plugin_functions_type *wp_plugin_functions) {
    af_return_code rc = af_code_error;
    if ( m_bspwaveWidget && wp_plugin_functions ) {
        af_wp_access_functions_type wp_functions;
        rc = win_bspwave_get_waveform_parser(m_bspwaveWidget, &wp_functions);
        if ( rc == af_code_ok ) {
            if ( wp_functions.register_plugin_functions && wp_functions.parser_object ) {
                rc = ( wp_functions.register_plugin_functions )( wp_functions.parser_object, wp_plugin_functions );
            }
        }
    }
    return rc;
}
af_return_code winBspWaveWrapperImpl::unregisterWaveformParserPlugin(struct _af_wp_plugin_functions_type *wp_plugin_functions) {
    af_return_code rc = af_code_error;
    if ( m_bspwaveWidget && wp_plugin_functions ) {
        af_wp_access_functions_type wp_functions;
        rc = win_bspwave_get_waveform_parser(m_bspwaveWidget, &wp_functions);
        if ( rc == af_code_ok ) {
            if ( wp_functions.unregister_plugin_functions && wp_functions.parser_object ) {
                rc = ( wp_functions.unregister_plugin_functions )( wp_functions.parser_object, wp_plugin_functions );
            }
        }
    }
    return rc;
}
af_return_code winBspWaveWrapperImpl::getWaveformParserAccessFunctions(struct _af_wp_access_functions_type *access_functions) {
    if ( m_bspwaveWidget && access_functions ) {
        if ( win_bspwave_get_waveform_parser(m_bspwaveWidget, access_functions) == af_code_ok ) {
            return af_code_ok;
        }
    }
    return af_code_error;
}

HWND winBspWaveWrapperImpl::getHandle(int m_info_type) {
    if ( m_bspwaveWidget ) {
        struct af_bspwave_win_widget_info_type widget_info;
        memset(&widget_info, 0, sizeof( af_bspwave_win_widget_info_type ));
        widget_info.m_info_type = m_info_type;
        win_bspwave_get_widget_info(m_bspwaveWidget, &widget_info);
        return widget_info.m_widget_handle;
    }
    return 0;
}

HWND winBspWaveWrapperImpl::getBrowserHandle() {
    return getHandle(AF_BSPWAVE_WIDGET_INFO_TYPE_BROWSER_HANDLE);
}

HWND winBspWaveWrapperImpl::getMeasurementsHandle() {
    return getHandle(AF_BSPWAVE_WIDGET_INFO_TYPE_MEASUREMENTS_HANDLE);
}

HWND winBspWaveWrapperImpl::getPageParametersHandle() {
    return getHandle(AF_BSPWAVE_WIDGET_INFO_TYPE_PAGE_PARAMETERS_HANDLE);
}

HWND winBspWaveWrapperImpl::getPageLayoutHandle() {
    return getHandle(AF_BSPWAVE_WIDGET_INFO_TYPE_LAYOUT_HANDLE);
}

HWND winBspWaveWrapperImpl::getSidebarHandle() {
    return getHandle(AF_BSPWAVE_WIDGET_INFO_TYPE_SIDEBAR_HANDLE);
}

HWND winBspWaveWrapperImpl::getToolbarHandle() {
    return getHandle(AF_BSPWAVE_WIDGET_INFO_TYPE_TOOLBAR_HANDLE);
}
HWND winBspWaveWrapperImpl::getTabCtrlHandle() {
    return getHandle(AF_BSPWAVE_WIDGET_INFO_TYPE_TAB_CTRL_HANDLE);
}







