/*
    Copyright Analog Flavor 2013-2024

    All rights reserved.

    This software contains proprietary and confidential information which is the
    property of Analog Flavor and is subject to license terms.

    This file may be used in accordance with the license agreement 
    provided with the software.

    This copyright notice may not be removed or altered.
*/


#include <stdafx.h>

#include "winBspWaveWrapperRuntime.h"
#include "bspwave_win_widget.h"
#include "bspwave_plugin.h"

#include "bspWaveWrapper.impl.h"
#include <sstream>


/// callback functions
extern "C" {

    void win_runtime_bsp_wave_wrapper_widget_destroyed(void *obj) {
        if ( obj ) {
            winBspWaveWrapperRuntimeImpl *wrapper = (winBspWaveWrapperRuntimeImpl*)( obj );
            bool flag_delete_object = false;
            wrapper->onWidgetDestroyed(flag_delete_object);
            if ( flag_delete_object ) {
                delete wrapper;
            }
        }
    }
    void win_runtime_bsp_wave_wrapper_curve_highlighted(void *obj, const char *section_name, const char *curve_name, int section_id, int curve_index, int flag_clear, int from_browser) {
        if ( obj && section_name && curve_name ) {
            winBspWaveWrapperRuntimeImpl *wrapper = (winBspWaveWrapperRuntimeImpl*)(obj);
            wrapper->onCurveHighlighted(std::string(section_name), std::string(curve_name), section_id, curve_index, flag_clear, from_browser);
        }
    }
    void win_runtime_bsp_wave_wrapper_curve_un_highlighted(void *obj, const char *section_name, const char *curve_name, int section_id, int curve_index, int flag_clear, int from_browser) {
        if ( obj && section_name && curve_name ) {
            winBspWaveWrapperRuntimeImpl *wrapper = (winBspWaveWrapperRuntimeImpl*)(obj);
            wrapper->onCurveUnHighlighted(std::string(section_name), std::string(curve_name), section_id, curve_index, flag_clear, from_browser);
        }
    }
    void win_runtime_bsp_wave_wrapper_curve_activated(void *obj, const char *section_name, const char *curve_name, int section_id, int curve_index, int flag_clear, int from_browser) {
        if ( obj && section_name && curve_name ) {
            winBspWaveWrapperRuntimeImpl *wrapper = (winBspWaveWrapperRuntimeImpl*)(obj);
            wrapper->onCurveActivated(std::string(section_name), std::string(curve_name), section_id, curve_index, flag_clear, from_browser);
        }
    }
    int win_runtime_bsp_wave_wrapper_notify(void *obj, const struct _af_bspwave_callback_notification_type *description) {
        if ( obj && description ) {
            winBspWaveWrapperRuntimeImpl *wrapper = (winBspWaveWrapperRuntimeImpl*)( obj );
            return wrapper->onNotify(description);
        }
        return 0;
    }

    const char *win_runtime_bsp_wave_wrapper_drop_received(void *obj, const char *plot_name, const char *received_text) {
        if ( obj && plot_name && received_text ) {
            winBspWaveWrapperRuntimeImpl *wrapper = (winBspWaveWrapperRuntimeImpl*)( obj );
            wrapper->tmpString() = std::string(received_text);
            if ( wrapper->onDropReceived(std::string(plot_name), wrapper->tmpString()) ) {
                return wrapper->tmpString().c_str();
            }
        }
        return 0;
    }
    const char *win_runtime_bsp_wave_wrapper_generate_drag_string(void *obj, const char *xml_drag_string) {
        if ( obj && xml_drag_string ) {
            winBspWaveWrapperRuntimeImpl *wrapper = (winBspWaveWrapperRuntimeImpl*)( obj );
            std::string xml_string(xml_drag_string);
            if ( wrapper->onGenerateDragString(xml_string, wrapper->tmpString()) ) {
                return wrapper->tmpString().c_str();
            }
        }
        return 0;
    }
    void win_runtime_bsp_wave_wrapper_check_integrator_licence(void *obj, void *data) {
        if ( obj ) {
            winBspWaveWrapperRuntimeImpl *wrapper = (winBspWaveWrapperRuntimeImpl*)(obj);
            wrapper->onCheckIntegratorLicence(data);
        }
    }
    void win_runtime_bsp_wave_wrapper_tab_ctrl_page_added(void *obj, const char *page_name, HWND page_handle) {
        if ( obj ) {
            winBspWaveWrapperRuntimeImpl *wrapper = (winBspWaveWrapperRuntimeImpl*)( obj );
            wrapper->onTabCtrlPageAdded(page_name, page_handle);
        }
    }
    void win_runtime_bsp_wave_wrapper_tab_ctrl_page_removed(void *obj, const char *page_name, HWND page_handle) {
        if ( obj ) {
            winBspWaveWrapperRuntimeImpl *wrapper = (winBspWaveWrapperRuntimeImpl*)( obj );
            wrapper->onTabCtrlPageRemoved(page_name, page_handle);
        }
    }
    void win_runtime_bsp_wave_wrapper_tab_ctrl_page_changed(void *obj, const char *page_name, HWND page_handle) {
        if ( obj ) {
            winBspWaveWrapperRuntimeImpl *wrapper = (winBspWaveWrapperRuntimeImpl*)( obj );
            wrapper->onTabCtrlPageChanged(page_name, page_handle);
        }
    }
    void win_runtime_bsp_wave_wrapper_ensure_page_visible(void *obj, const char *page_name, HWND page_handle) {
        if ( obj ) {
            winBspWaveWrapperRuntimeImpl *wrapper = (winBspWaveWrapperRuntimeImpl*)( obj );
            wrapper->onEnsurePageVisible(page_name, page_handle);
        }
    }

} /// extern c





winBspWaveWrapperRuntimeImpl::winBspWaveWrapperRuntimeImpl() {
    m_bspwaveWidget = 0;
    m_libraryHandle = 0;
    m_flagDestroyObjectWhenClosing = false;
    clearFunctionPointers();
}
winBspWaveWrapperRuntimeImpl::~winBspWaveWrapperRuntimeImpl() {
}
void winBspWaveWrapperRuntimeImpl::clearFunctionPointers() {
    m_createWidget = 0;
    m_processCommandLine = 0;
    m_executeCommand = 0;
    m_getCommandResult = 0;
    m_registerCallback = 0;
    m_createAndRunMain = 0;
    m_getWaveformParser = 0;
    m_getWidgetInfo = 0;
    m_registerWinCallback = 0;
    m_registerStyleCallback = 0;
}
void winBspWaveWrapperRuntimeImpl::loadBspWaveDll(const wchar_t *library_path) {
    // load library
    if ( m_libraryHandle == 0 && library_path ) {
        m_libraryHandle = ::LoadLibraryW(library_path);
    }
    loadFunctionPointers();
}
void winBspWaveWrapperRuntimeImpl::loadBspWaveDll(const char *library_path) {
    // load library
    if ( m_libraryHandle == 0 && library_path ) {
        m_libraryHandle = ::LoadLibraryA(library_path);
    }
    loadFunctionPointers();
}
void winBspWaveWrapperRuntimeImpl::loadFunctionPointers() {
    // get function pointers
    if ( m_libraryHandle ) {
        m_createWidget = (win_bspwave_create_widget_type*)(::GetProcAddress((HMODULE)m_libraryHandle, "win_bspwave_create_widget"));
        m_processCommandLine = (win_bspwave_process_command_line_type*)(::GetProcAddress((HMODULE)m_libraryHandle, "win_bspwave_process_command_line"));
        m_executeCommand = (win_bspwave_execute_command_type*)( ::GetProcAddress((HMODULE)m_libraryHandle, "win_bspwave_execute_command") );
        m_getCommandResult = (win_bspwave_get_command_result_type*)( ::GetProcAddress((HMODULE)m_libraryHandle, "win_bspwave_get_command_result") );
        m_registerCallback = (win_bspwave_register_callback_type*)( ::GetProcAddress((HMODULE)m_libraryHandle, "win_bspwave_register_callback") );
        m_createAndRunMain = (win_bspwave_create_and_run_main_type*)( ::GetProcAddress((HMODULE)m_libraryHandle, "win_bspwave_create_and_run_main") );
        m_getWaveformParser = (win_bspwave_get_waveform_parser_type*)( ::GetProcAddress((HMODULE)m_libraryHandle, "win_bspwave_get_waveform_parser") );
        m_getWidgetInfo = (win_bspwave_get_widget_info_type*)( ::GetProcAddress((HMODULE)m_libraryHandle, "win_bspwave_get_widget_info") );
        m_registerWinCallback = (win_bspwave_register_win_callback_type*)( ::GetProcAddress((HMODULE)m_libraryHandle, "win_bspwave_register_win_callback") );
        m_registerStyleCallback = (win_bspwave_register_style_callback_type*)( ::GetProcAddress((HMODULE)m_libraryHandle, "win_bspwave_register_style_callback") );
    }
    else {
        clearFunctionPointers();
    }
}
bool winBspWaveWrapperRuntimeImpl::createWidget(HWND parent, int style_flags) {
    return  createWidget(parent, 0, 0, 100, 100, style_flags);
}
bool winBspWaveWrapperRuntimeImpl::createWidget(HWND parent, int x, int y, int w, int h, int style_flags) {
    if (m_bspwaveWidget || ! m_createWidget) {
        return false;
    }
    m_bspwaveWidget = m_createWidget(parent, x, y, w, h, style_flags);
    if ( m_bspwaveWidget ) {
        registerCallbackFunctions(false, false);
        return true;
    }
    return false;
}
bool winBspWaveWrapperRuntimeImpl::createMainWindow(int style_flags) {
    if ( m_bspwaveWidget || ! m_createAndRunMain ) {
        return false;
    }
    m_bspwaveWidget = m_createAndRunMain(-1, -1, style_flags, true, 0);
    if (m_bspwaveWidget) {
        registerCallbackFunctions(false, false);
        return true;
    }
    return false;
}
void winBspWaveWrapperRuntimeImpl::setGeometry(int x, int y, int w, int h) {
    if ( m_bspwaveWidget ) {
        ::MoveWindow(m_bspwaveWidget, x, y, w, h, TRUE);
    }
}
af_return_code winBspWaveWrapperRuntimeImpl::cmdStd(const std::string &cmd) {
    af_return_code rc = af_code_error;
    if ( m_bspwaveWidget && m_executeCommand ) {
        const char *c_cmd = cmd.c_str();
        rc = m_executeCommand(m_bspwaveWidget, c_cmd);
    }
    return rc;
}
af_return_code winBspWaveWrapperRuntimeImpl::cmdClose(bool flag_force) {
    af_return_code rc = af_code_error;
    if ( flag_force ) {
        /// files in text editor might not be saved
        rc = cmdStd(std::string("close 1"));
    }
    else {
        rc = cmdStd(std::string("close 0"));
    }
    return rc;
}
const char **winBspWaveWrapperRuntimeImpl::getCommandResult() {
    if ( m_bspwaveWidget && m_getCommandResult ) {
        return m_getCommandResult(m_bspwaveWidget);
    }
    return 0;
}
af_return_code winBspWaveWrapperRuntimeImpl::processCommandLine(LPSTR cmd) {
    af_return_code rc = af_code_error;
    if (m_bspwaveWidget && m_processCommandLine ) {
        rc = m_processCommandLine(m_bspwaveWidget, cmd);
    }
    return rc;
}
void winBspWaveWrapperRuntimeImpl::registerCallbackFunctions(bool flag_drag_string_gene, bool flag_cursor_values) {
    af_bspwave_callback_type callbacks;
    memset(&callbacks, 0, sizeof(af_bspwave_callback_type));

    callbacks.callback_object = this;
    callbacks.widget_destroyed = win_runtime_bsp_wave_wrapper_widget_destroyed;
    callbacks.curve_highlighted = win_runtime_bsp_wave_wrapper_curve_highlighted;
    callbacks.curve_un_highlighted = win_runtime_bsp_wave_wrapper_curve_un_highlighted;
    callbacks.curve_activated = win_runtime_bsp_wave_wrapper_curve_activated;
    callbacks.notify = win_runtime_bsp_wave_wrapper_notify;
    callbacks.drop_received = win_runtime_bsp_wave_wrapper_drop_received;
    if ( flag_drag_string_gene ) {
        callbacks.generate_drag_string = win_runtime_bsp_wave_wrapper_generate_drag_string;
    }
    callbacks.check_integrator_licence = win_runtime_bsp_wave_wrapper_check_integrator_licence;

    if ( m_bspwaveWidget && m_registerCallback ) {
        m_registerCallback(m_bspwaveWidget, &callbacks);
    }
}
void winBspWaveWrapperRuntimeImpl::unregisterCallbackFunctions() {
    if ( m_bspwaveWidget && m_registerCallback ) {
        af_bspwave_callback_type callbacks;
        memset(&callbacks, 0, sizeof( af_bspwave_callback_type ));
        m_registerCallback(m_bspwaveWidget, &callbacks);
    }
}
af_return_code winBspWaveWrapperRuntimeImpl::registerWidgetCallbackFunctions() {
    if ( m_bspwaveWidget && m_registerWinCallback ) {
        af_bspwave_win_callback_type callbacks;
        memset(&callbacks, 0, sizeof( af_bspwave_win_callback_type ));
        callbacks.callback_object = this;
        callbacks.tab_ctrl_page_added = win_runtime_bsp_wave_wrapper_tab_ctrl_page_added;
        callbacks.tab_ctrl_page_changed = win_runtime_bsp_wave_wrapper_tab_ctrl_page_changed;
        callbacks.tab_ctrl_page_removed = win_runtime_bsp_wave_wrapper_tab_ctrl_page_removed;
        callbacks.ensure_page_visible = win_runtime_bsp_wave_wrapper_ensure_page_visible;
        return m_registerWinCallback(m_bspwaveWidget, &callbacks);
    }
    return af_code_error;
}
af_return_code winBspWaveWrapperRuntimeImpl::unregisterWidgetCallbackFunctions() {
    if ( m_bspwaveWidget && m_registerWinCallback ) {
        af_bspwave_win_callback_type callbacks;
        memset(&callbacks, 0, sizeof( af_bspwave_win_callback_type ));
        return m_registerWinCallback(m_bspwaveWidget, &callbacks);
    }
    return af_code_error;
}
af_return_code winBspWaveWrapperRuntimeImpl::registerWaveformParserPlugin(struct _af_wp_plugin_functions_type *wp_plugin_functions) {
    af_return_code rc = af_code_error;
    if ( m_bspwaveWidget && wp_plugin_functions && m_getWaveformParser ) {
        af_bspwave_plugin_waveform_parser_type wp_functions;
        rc = m_getWaveformParser(m_bspwaveWidget, &wp_functions);
        if ( rc == af_code_ok ) {
            if ( wp_functions.register_plugin_functions && wp_functions.parser_object ) {
                rc = ( wp_functions.register_plugin_functions )( wp_functions.parser_object, wp_plugin_functions );
            }
        }
    }
    return rc;
}
af_return_code winBspWaveWrapperRuntimeImpl::unregisterWaveformParserPlugin(struct _af_wp_plugin_functions_type *wp_plugin_functions) {
    af_return_code rc = af_code_error;
    if ( m_bspwaveWidget && wp_plugin_functions && m_getWaveformParser ) {
        af_bspwave_plugin_waveform_parser_type wp_functions;
        rc = m_getWaveformParser(m_bspwaveWidget, &wp_functions);
        if ( rc == af_code_ok ) {
            if ( wp_functions.unregister_plugin_functions && wp_functions.parser_object ) {
                rc = ( wp_functions.unregister_plugin_functions )( wp_functions.parser_object, wp_plugin_functions );
            }
        }
    }
    return rc;
}
int winBspWaveWrapperRuntimeImpl::onNotify(const struct _af_bspwave_callback_notification_type *description) {
    return handleNotification(description);
}
int winBspWaveWrapperRuntimeImpl::handleNotification(const struct _af_bspwave_callback_notification_type *description) {
    std::string message;
    if ( description->m_string_value ) {
        message = std::string(description->m_string_value);
    }
    switch ( description->m_type ) {
    case af_bspwave_notification_logger_message:
        onLoggerMessage(description->m_integer_value, message);
        break;
    case af_bspwave_notification_status_bar_message:
        onStatusBarMessage(description->m_integer_value, message);
        break;
    case af_bspwave_notification_progress_message:
        onProgressMessage(message, description->m_integer_value);
        break;
    case af_bspwave_notification_file_opened:
        onFileWasOpened(message);
        break;
    case af_bspwave_notification_file_closed:
        onFileWasClosed(message);
        break;
    case af_bspwave_notification_all_files_closed:
        onAllFilesWereClosed();
        break;
    case af_bspwave_notification_section_used_for_link_changed:
        onSectionUsedForLinkChanged();
        break;
    case af_bspwave_notification_close_button_activated:
        onCloseButtonActivated();
        break;
    case af_bspwave_notification_attach_button_activated:
        onAttachButtonActivated();
        break;
    case af_bspwave_notification_detach_button_activated:
        onDetachButtonActivated();
        break;
    case af_bspwave_notification_open_file_button_activated:
        return onOpenFileButtonActivated();
    default:
        break;
    }
    return 0;
}




