/*
    Copyright Analog Flavor 2013-2024

    All rights reserved.

    This software contains proprietary and confidential information which is the
    property of Analog Flavor and is subject to license terms.

    This file may be used in accordance with the license agreement 
    provided with the software.

    This copyright notice may not be removed or altered.
*/


#include "wxWinBspWaveWrapper.h"

#include <windows.h>
#include "bspwave_win_widget.h"
#include "bspWaveWrapper.impl.h"


template class bspWaveWrapper<wxWinBspWaveWrapperImpl>;



wxWinBspWaveWrapperImpl::wxWinBspWaveWrapperImpl(wxWindow *parent)
    : wxWindow(parent, wxID_ANY) {

    HWND parent_hwnd = (HWND)(wxWindow::GetHWND());

    m_hwnd = 0;
    if ( parent_hwnd ) {
        HWND bspwave_hwnd = win_bspwave_create_widget(parent_hwnd, 0, 0, 100, 100,
                                              AF_BSPWAVE_STYLE_DEBUG
                                              | AF_BSPWAVE_STYLE_USE_TOOLBAR
                                              | AF_BSPWAVE_STYLE_USE_NOTEBOOK);

        m_hwnd = (int)bspwave_hwnd;
    }
    this->Connect( wxEVT_SIZE, wxSizeEventHandler( wxWinBspWaveWrapperImpl::on_size ) );
}

wxWinBspWaveWrapperImpl::~wxWinBspWaveWrapperImpl() {
    this->Disconnect( wxEVT_SIZE, wxSizeEventHandler( wxWinBspWaveWrapperImpl::on_size ) );
}

void wxWinBspWaveWrapperImpl::on_size( wxSizeEvent& event ) {
    int width, height;
    if ( m_hwnd != 0 ) {
        HWND bspwave_hwnd = (HWND)m_hwnd;
        wxSize s = event.GetSize();
        width = s.GetWidth();
        height = s.GetHeight();
        win_bspwave_move_widget(bspwave_hwnd, 0, 0, width, height);
    }
}
af_return_code wxWinBspWaveWrapperImpl::cmdStd(const std::string &cmd) {
    af_return_code rc = af_code_error;
    if ( m_hwnd != 0 ) {
        rc = win_bspwave_execute_command((HWND)m_hwnd, cmd.c_str());
    }
    return rc;
}
af_return_code wxWinBspWaveWrapperImpl::cmdWx(const wxString &str_cmd) {
    af_return_code rc = af_code_error;
    if ( m_hwnd != 0 ) {
#if wxUSE_UNICODE_WCHAR
        /// we must not pass objects such as strings over dll borderies (windows)
        const wchar_t *wc = str_cmd.wc_str();
        char s[1000];
        size_t l;
        l = WideCharToMultiByte(CP_ACP, 0, wc, -1, s, 1000, NULL, NULL);
#else
        const char* s = 0;
        s = cmd_line.mb_str();
#endif
        rc = win_bspwave_execute_command((HWND)m_hwnd, s);
    }
    return rc;
}
af_return_code wxWinBspWaveWrapperImpl::cmdWx(const wxString &str_cmd, wxArrayString &string_results) {
    const char **bsp_result;
    const char *str;
    wxString wxs;

    af_return_code rc = cmdWx(str_cmd);
    
    string_results.Clear();
    if ( rc == af_code_ok ) {
        bsp_result = win_bspwave_get_command_result((HWND)m_hwnd);
        if ( bsp_result ) {
            while ( *bsp_result ) {
                str = *bsp_result;
                wxs = wxString::FromAscii(str);
                string_results.Add(wxs);
                bsp_result++;
            }
        }
    }
    return rc;
}
af_return_code wxWinBspWaveWrapperImpl::processCommandLineWx(const wxString &cmd_line) {
    af_return_code rc = af_code_error;
    if (m_hwnd != 0) {
#if wxUSE_UNICODE_WCHAR
        /// we must not pass objects such as strings over dll borderies (windows)
        const wchar_t *wc = cmd_line.wc_str();
        char s[1000];
        size_t l;
        l = WideCharToMultiByte(CP_ACP, 0, wc, -1, s, 1000, NULL, NULL);
#else
        const char* s = 0;
        s = cmd_line.mb_str();
#endif
        rc = win_bspwave_process_command_line((HWND)m_hwnd, s);
    }
    return rc;
}
af_return_code wxWinBspWaveWrapperImpl::processCommandLineStd(const std::string &cmd_line) {
    af_return_code rc = af_code_error;
    if (m_hwnd != 0) {
        rc = win_bspwave_process_command_line((HWND)m_hwnd, (LPSTR)(cmd_line.c_str()));
    }
    return rc;
}
const char **wxWinBspWaveWrapperImpl::getCommandResult() {
    if ( m_hwnd != 0 ) {
        return win_bspwave_get_command_result((HWND)m_hwnd);
    }
    return 0;
}


