/*
    Copyright Analog Flavor 2013-2024

    All rights reserved.

    This software contains proprietary and confidential information which is the
    property of Analog Flavor and is subject to license terms.

    This file may be used in accordance with the license agreement 
    provided with the software.

    This copyright notice may not be removed or altered.
*/

/*
#
#  Copyright Analog Flavor and Gerard Eichenmueller 2013
#
#  All rights reserved.
#
#  This software contains proprietary and confidential information which is the
#  property of Analog Flavor and Gerard Eichenmueller and is subject to license terms.
#
*/

#include <fstream>
#include <sstream>
#include <iostream>
#include <string>
#include <string.h>
using namespace std;

#include <tcl.h>
#include <tk.h>

#include "bspwave_tk_win_widget.h"

static Tk_Window bspwave_widget_test_window = 0;
static int application_argc = 0;
static char **application_argv = 0;

int bspwave_main(Tcl_Interp *interp, char *window_path) {
    std::string cmd;
    std::stringstream cmd_str;

    if ( win_tk_bspwave_initialize_library(interp) != TCL_OK ) {
        cout << "Failed to initialize library." << endl;
        return TCL_ERROR;
    }

    Tk_Window tk_parent = Tk_MainWindow(interp);

    int style = AF_BSPWAVE_STYLE_USE_TOOLBAR 
        | AF_BSPWAVE_STYLE_USE_NOTEBOOK 
        | AF_BSPWAVE_STYLE_AUTO_LOAD_SAVE_CONFIG
        | AF_BSPWAVE_STYLE_DEBUG;
    bspwave_widget_test_window = win_tk_bspwave_create_widget_from_path(interp, tk_parent, window_path, 10, 50, 600, 500, style);

    if (bspwave_widget_test_window == NULL) {
        return TCL_ERROR;
    }

    const char *bspwave_window_name = Tk_PathName(bspwave_widget_test_window);
    std::string bspwave_name;
    if ( bspwave_window_name ) {
        bspwave_name = std::string(bspwave_window_name);
    }

    win_tk_bspwave_process_command_line(bspwave_widget_test_window, application_argc, application_argv);

    cmd_str << "menu .menu -tearoff 0" << endl;
    cmd_str << "menu .menu.file -tearoff 0" << endl;
    cmd_str << "menu .menu.dev -tearoff 0" << endl;

    cmd_str << ".menu add cascade -label \"File\" -menu .menu.file -underline 0" << endl;
    cmd_str << ".menu.file add command -label \"Open file\" -command {" << endl;
    cmd_str << "    set filename [tk_getOpenFile -title \"Open Waveform File\"]" << endl;
    cmd_str << "    if {$filename ne \"\"} {" << endl;
    cmd_str << "        " << bspwave_name << " execute_command \"open_file $filename\"" << endl;
    cmd_str << "    }" << endl;
    cmd_str << "}" << endl;

    cmd_str << ".menu add cascade -label \"Dev\" -menu .menu.dev -underline 0" << endl;
    cmd_str << ".menu.dev add command -label \"Show Control Panel\" -command {" << endl;
    cmd_str << "    " << bspwave_name << " execute_command \"show_control_panel 1\"" << endl;
    cmd_str << "}" << endl;
    cmd_str << ".menu.dev add command -label \"Show Command Logger\" -command {" << endl;
    cmd_str << "    " << bspwave_name << " execute_command \"show_command_logger 1\"" << endl;
    cmd_str << "}" << endl;
    cmd_str << ".menu.dev add command -label \"Hide Panels\" -command {" << endl;
    cmd_str << "    " << bspwave_name << " execute_command \"show_control_panel 0\"" << endl;
    cmd_str << "    " << bspwave_name << " execute_command \"show_command_logger 0\"" << endl;
    cmd_str << "}" << endl;

    cmd_str << ". configure -menu .menu" << endl;
    Tcl_Eval(interp, cmd_str.str().c_str());

    cmd = "pack " + std::string(window_path) + " -padx 10 -pady 10 -side left -expand yes -fill both";
    Tcl_Eval(interp, cmd.c_str());

    cmd = std::string(window_path) + " execute_command \"load_available_plugins\"";
    Tcl_Eval(interp, cmd.c_str());

    return TCL_OK;
}

int init_proc( Tcl_Interp *interp ) {

    int rc;
    rc = Tcl_Init( interp );
    if( rc != TCL_OK) {
        return rc;
    }
    rc = Tk_Init( interp );
    if( rc != TCL_OK) {
        return rc;
    }

    std::string window_name = ".test_bspwave_win_tk";
    rc = bspwave_main(interp, (char*)(window_name.c_str()));

    return rc;
}


int main(int argc, char **argv) {


    application_argc = argc;
    application_argv = argv;
    Tk_Main( argc, argv, init_proc );
    return 0;
}

