/*
    Copyright Analog Flavor 2013-2024

    All rights reserved.

    This software contains proprietary and confidential information which is the
    property of Analog Flavor and is subject to license terms.

    This file may be used in accordance with the license agreement 
    provided with the software.

    This copyright notice may not be removed or altered.
*/


#include <windows.h>
#include <Commctrl.h>

#include "bspwave_win_widget.h"

#define ID_MENU_FILE_MENU 998
#define ID_MENU_DEV_MENU 999
#define ID_MENU_OPEN_FILE 1000
#define ID_MENU_SHOW_PANEL 1001
#define ID_MENU_LOGGER 1002
#define ID_MENU_HIDE_PANELS 1003
#define ID_MENU_CLOSE 1004

const TCHAR g_szClassName[] = TEXT("myWindowClass");

static HINSTANCE h_instance = 0;
static HWND bspwave_hwnd = 0;
static HWND main_hwnd = 0;
static HMENU main_menu = 0;
static const char *bspwave_command_line_arguments = 0;

void create_menu(HWND hwnd) {
    MENUITEMINFO menu_info;
    HMENU file_menu, dev_menu;

    menu_info.cbSize = sizeof(MENUITEMINFO);

    if ( main_menu == 0 && hwnd != 0 ) {

        main_menu = CreateMenu();
        file_menu = CreateMenu();
        dev_menu = CreateMenu();

        menu_info.fMask = MIIM_FTYPE | MIIM_STRING | MIIM_SUBMENU;
        menu_info.fType = MFT_STRING;
        menu_info.dwTypeData = "File";
        menu_info.hSubMenu = file_menu;
        InsertMenuItem(main_menu, ID_MENU_FILE_MENU, FALSE, &menu_info);

        menu_info.dwTypeData = "Dev";
        menu_info.hSubMenu = dev_menu;
        InsertMenuItem(main_menu, ID_MENU_FILE_MENU, FALSE, &menu_info);

        menu_info.fMask = MIIM_FTYPE | MIIM_STATE | MIIM_ID | MIIM_STRING;
        menu_info.fType = MFT_STRING;
        menu_info.fState = MFS_ENABLED;

        menu_info.wID = ID_MENU_OPEN_FILE;
        menu_info.dwTypeData = "Open File";
        InsertMenuItem(file_menu, ID_MENU_OPEN_FILE, FALSE, &menu_info);

        menu_info.wID = ID_MENU_CLOSE;
        menu_info.dwTypeData = "Close";
        InsertMenuItem(file_menu, ID_MENU_CLOSE, FALSE, &menu_info);

        menu_info.wID = ID_MENU_SHOW_PANEL;
        menu_info.dwTypeData = "Show Control Panel";
        InsertMenuItem(dev_menu, ID_MENU_SHOW_PANEL, FALSE, &menu_info);

        menu_info.wID = ID_MENU_LOGGER;
        menu_info.dwTypeData = "Show Command Logger";
        InsertMenuItem(dev_menu, ID_MENU_LOGGER, FALSE, &menu_info);

        menu_info.wID = ID_MENU_HIDE_PANELS;
        menu_info.dwTypeData = "Hide Panels";
        InsertMenuItem(dev_menu, ID_MENU_HIDE_PANELS, FALSE, &menu_info);

        SetMenu(hwnd, main_menu);
    }
}
void create_bspwave_widget(HWND hwnd) {
    RECT rect;
    int w, h;
    if ( ! bspwave_hwnd ) {
        w = 100;
        h = 100;
        if ( GetClientRect(hwnd, &rect) ) {
            w = rect.right - rect.left;
            h = rect.bottom - rect.top;
        }
        bspwave_hwnd = win_bspwave_create_widget(hwnd, 0, 0, w, h,
            AF_BSPWAVE_STYLE_DEBUG | AF_BSPWAVE_STYLE_USE_TOOLBAR | AF_BSPWAVE_STYLE_USE_NOTEBOOK);
        win_bspwave_process_command_line(bspwave_hwnd, bspwave_command_line_arguments);

    }
}
void update_layout(HWND hwnd) {
    RECT rect;
    int w, h;
    if ( hwnd == main_hwnd ) {
        if ( GetClientRect(hwnd, &rect) ) {
            w = rect.right - rect.left;
            h = rect.bottom - rect.top;
            if ( bspwave_hwnd ) {
                MoveWindow(bspwave_hwnd, 0, 0, w, h, TRUE);
            }
        }
    }
}


// the Window Procedure
LRESULT CALLBACK WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
    OPENFILENAME dlg_info;
    TCHAR file_name[512];
    size_t max_file_name_size = 512;

    switch (msg)
    {
        case WM_CREATE:
            create_menu(hwnd);
            create_bspwave_widget(hwnd);
            break;
        case WM_CLOSE:
            DestroyWindow(hwnd);
            break;
        case WM_DESTROY:
            PostQuitMessage(0);
            break;
        case WM_SIZE:
            switch ( wParam ) {
            case SIZE_MAXIMIZED:
            case SIZE_MINIMIZED:
            case SIZE_RESTORED:
                update_layout(hwnd);
                break;
            default:
                break;
            }
            break;
        case WM_COMMAND:
            switch (LOWORD(wParam)) {
            case ID_MENU_OPEN_FILE:
                memset(&dlg_info, 0, sizeof(OPENFILENAME));
                dlg_info.lStructSize = sizeof(OPENFILENAME);
                dlg_info.hwndOwner = hwnd;
                dlg_info.lpstrFile = file_name;
                dlg_info.nMaxFile = max_file_name_size;
                file_name[0] = '\0';
                if (GetOpenFileName(&dlg_info)) {
                    TCHAR bsp_cmd[1000];
                    strcpy(bsp_cmd, "open_file \"");
                    strcat(bsp_cmd, file_name);
                    strcat(bsp_cmd, "\"");
                    win_bspwave_execute_command(bspwave_hwnd, bsp_cmd);
                }
                break;
            case ID_MENU_SHOW_PANEL:
                win_bspwave_execute_command(bspwave_hwnd, "show_control_panel 1");
                break;
            case ID_MENU_LOGGER:
                win_bspwave_execute_command(bspwave_hwnd, "show_command_logger 1");
                break;
            case ID_MENU_HIDE_PANELS:
                win_bspwave_execute_command(bspwave_hwnd, "show_control_panel 0");
                win_bspwave_execute_command(bspwave_hwnd, "show_command_logger 0");
                break;
            case ID_MENU_CLOSE:
                win_bspwave_execute_command(bspwave_hwnd, "close");
                bspwave_hwnd = 0;
                break;
            }
            break;
        default:
            return DefWindowProc(hwnd, msg, wParam, lParam);
    }
    return 0;
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
    LPSTR lpCmdLine, int nCmdShow)
{
    WNDCLASSEX wc;
    MSG Msg;
    int w, h, nb_h;

    h_instance = hInstance;

    //Step 1: Registering the Window Class
    wc.cbSize        = sizeof(WNDCLASSEX);
    wc.style         = 0;
    wc.lpfnWndProc   = WndProc;
    wc.cbClsExtra    = 0;
    wc.cbWndExtra    = 0;
    wc.hInstance     = hInstance;
    wc.hIcon         = LoadIcon(NULL, IDI_APPLICATION);
    wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
    wc.lpszMenuName  = NULL;
    wc.lpszClassName = g_szClassName;
    wc.hIconSm       = LoadIcon(NULL, IDI_APPLICATION);

    if(!RegisterClassEx(&wc))
    {
        MessageBox(NULL, TEXT("Window Registration Failed!"), TEXT("Error!"),
            MB_ICONEXCLAMATION | MB_OK);
        return 0;
    }

    // Step 2: Creating the Window
    w = 700;
    h = 620;
    nb_h = h / 2;
    bspwave_command_line_arguments = lpCmdLine;
    main_hwnd = CreateWindowEx(
        0,
        g_szClassName,
        TEXT("BspWave Widget Test"),
        WS_OVERLAPPEDWINDOW,
        CW_USEDEFAULT, CW_USEDEFAULT, w, h,
        NULL, NULL, hInstance, NULL);

    if(main_hwnd == NULL)
    {
        MessageBox(NULL, TEXT("Window Creation Failed!"), TEXT("Error!"),
            MB_ICONEXCLAMATION | MB_OK);
        return 0;
    }

    ShowWindow(main_hwnd, nCmdShow);
    UpdateWindow(main_hwnd);

    update_layout(main_hwnd);

    // Step 3: The Message Loop
    while(GetMessage(&Msg, NULL, 0, 0) > 0)
    {
        TranslateMessage(&Msg);
        DispatchMessage(&Msg);
    }
    return Msg.wParam;
}

