/*
    Copyright Analog Flavor 2013-2024

    All rights reserved.

    This software contains proprietary and confidential information which is the
    property of Analog Flavor and is subject to license terms.

    This file may be used in accordance with the license agreement 
    provided with the software.

    This copyright notice may not be removed or altered.
*/

#include "wxWinBspWaveTestApp.h"

#include "wxWinBspWaveWrapper.h"

#include <wx/frame.h>
#include <wx/sizer.h>
#include <wx/menu.h>
#include <wx/filedlg.h>

DECLARE_APP(wxWinBspWaveTestApp)
IMPLEMENT_APP(wxWinBspWaveTestApp)

#define ID_MENU_OPEN_FILE 1000
#define ID_MENU_SHOW_PANEL 1001
#define ID_MENU_LOGGER 1002
#define ID_MENU_HIDE_PANELS 1003

class wxWinBspWaveTestFrame : public wxFrame {

    wxWinBspWaveWrapper *m_bspWrapper;

public:

    wxWinBspWaveTestFrame();
    ~wxWinBspWaveTestFrame();

    void on_openFile( wxCommandEvent& event );
    void on_showPanel( wxCommandEvent& event );
    void on_showLogger( wxCommandEvent& event );
    void on_hidePanels( wxCommandEvent& event );

    void processCommandLineWx(wxString &cmd_line);
};

wxWinBspWaveTestFrame::wxWinBspWaveTestFrame()
: wxFrame(NULL, wxID_ANY, wxT("Example use of BspWave widget for WX on Windows"), wxDefaultPosition, wxSize(600, 500)) {

    /// create menu bar
    wxMenuBar *frame_menubar = new wxMenuBar( 0 );
    wxMenu *menu_file = new wxMenu();
    wxMenuItem *menu_item = new wxMenuItem( menu_file, ID_MENU_OPEN_FILE, wxT("Open File") , wxEmptyString, wxITEM_NORMAL );
    menu_file->Append( menu_item );
    frame_menubar->Append( menu_file, wxT("File") );

    wxMenu *menu_dev = new wxMenu();
    menu_item = new wxMenuItem( menu_dev, ID_MENU_SHOW_PANEL, wxT("Show Control Panel") , wxEmptyString, wxITEM_NORMAL  );
    menu_dev->Append( menu_item );
    menu_item = new wxMenuItem( menu_dev, ID_MENU_LOGGER, wxT("Show Command Logger") , wxEmptyString, wxITEM_NORMAL  );
    menu_dev->Append( menu_item );
    menu_item = new wxMenuItem( menu_dev, ID_MENU_HIDE_PANELS, wxT("Hide Panels") , wxEmptyString, wxITEM_NORMAL  );
    menu_dev->Append( menu_item );
    frame_menubar->Append( menu_dev, wxT("Dev") );

    SetMenuBar( frame_menubar );

    /// create bespice wave wrapper widget
    m_bspWrapper = new wxWinBspWaveWrapper(this);

    /// create sizer to automatically adjust the size of the wrapper widget
    wxBoxSizer *frame_sizer = new wxBoxSizer(wxVERTICAL);
    frame_sizer->Add(m_bspWrapper, 1, wxEXPAND | wxALL, 0);

    SetSizer(frame_sizer);
    Layout();

    /// connect event handler
    this->Connect( ID_MENU_OPEN_FILE, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( wxWinBspWaveTestFrame::on_openFile ) );
    this->Connect( ID_MENU_SHOW_PANEL, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( wxWinBspWaveTestFrame::on_showPanel ) );
    this->Connect( ID_MENU_LOGGER, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( wxWinBspWaveTestFrame::on_showLogger ) );
    this->Connect( ID_MENU_HIDE_PANELS, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( wxWinBspWaveTestFrame::on_hidePanels ) );

 //   m_bspWrapper->cmd("show_control_panel 1");
//    wx_bspwave_execute_command(bsp_frame, "show_command_logger 1");
}

wxWinBspWaveTestFrame::~wxWinBspWaveTestFrame() {
    this->Disconnect( ID_MENU_OPEN_FILE, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( wxWinBspWaveTestFrame::on_openFile ) );
    this->Disconnect( ID_MENU_SHOW_PANEL, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( wxWinBspWaveTestFrame::on_showPanel ) );
    this->Disconnect( ID_MENU_LOGGER, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( wxWinBspWaveTestFrame::on_showLogger ) );
    this->Disconnect( ID_MENU_HIDE_PANELS, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( wxWinBspWaveTestFrame::on_hidePanels ) );
}

void wxWinBspWaveTestFrame::on_showLogger( wxCommandEvent& event ) {
    m_bspWrapper->cmdShowCommandLogger();
}

void wxWinBspWaveTestFrame::on_hidePanels( wxCommandEvent& event ) {
    m_bspWrapper->cmdHidePanels();
}

void wxWinBspWaveTestFrame::on_showPanel( wxCommandEvent& event ) {
    m_bspWrapper->cmdShowControlPanel();
}

void wxWinBspWaveTestFrame::on_openFile( wxCommandEvent& event ) {
    wxArrayString string_results;
    af_return_code return_code;

    wxFileDialog dialog(this, wxT("Open File"), wxT(""), wxT(""), wxT(""), wxFD_OPEN);

    if ( dialog.ShowModal() == wxID_OK ) {
        wxString cmd = wxT("open_file ") + dialog.GetPath();
        return_code = m_bspWrapper->cmdWx(cmd, string_results);
    }
}

void wxWinBspWaveTestFrame::processCommandLineWx(wxString &cmd_line) {
    if (m_bspWrapper) {
        m_bspWrapper->processCommandLineWx(cmd_line);
    }
}


bool wxWinBspWaveTestApp::OnInit() {

    wxWinBspWaveTestFrame *bsp_frame = new wxWinBspWaveTestFrame();

    wxString cmd_line;
    wxString arg_line;

    int i = 1;
    while (i < this->argc) {
        arg_line = this->argv[i];
        cmd_line = cmd_line + wxString(" ") + arg_line;
        i++;
    }
    bsp_frame->processCommandLineWx(cmd_line);

    bsp_frame->Show();
    SetTopWindow(bsp_frame);

    return true;
}

